#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
This script is used to test Salt from a Jenkins server, specifically
jenkins.saltstack.com.

This script is intended to be shell-centric!!
"""

# Import python libs
from __future__ import absolute_import, print_function

import glob
import optparse
import os
import random
import re
import shutil
import subprocess
import sys
import time

# Import Salt libs
import salt.utils.files
import salt.utils.json
import salt.utils.stringutils
import salt.utils.yaml

try:
    from salt.utils.nb_popen import NonBlockingPopen
except ImportError:
    # Salt not installed, or nb_popen was not yet shipped with it
    SALT_LIB = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
    if SALT_LIB not in sys.path:
        sys.path.insert(0, SALT_LIB)
    try:
        # Let's try using the current checked out code
        from salt.utils.nb_popen import NonBlockingPopen
    except ImportError:
        # Still an ImportError??? Let's use some "brute-force"
        sys.path.insert(0, os.path.join(SALT_LIB, "salt", "utils"))
        from nb_popen import NonBlockingPopen

# Import 3rd-party libs
try:
    import requests

    HAS_REQUESTS = True
except ImportError:
    HAS_REQUESTS = False

SALT_GIT_URL = "https://github.com/saltstack/salt.git"


def build_pillar_data(options):
    """
    Build a YAML formatted string to properly pass pillar data
    """
    pillar = {
        "test_transport": options.test_transport,
        "cloud_only": options.cloud_only,
        "with_coverage": options.test_without_coverage is False,
    }
    if options.test_git_commit is not None:
        pillar["test_git_commit"] = options.test_git_commit
    if options.test_git_url is not None:
        pillar["test_git_url"] = options.test_git_url
    if options.bootstrap_salt_url is not None:
        pillar["bootstrap_salt_url"] = options.bootstrap_salt_url
    if options.bootstrap_salt_commit is not None:
        pillar["bootstrap_salt_commit"] = options.bootstrap_salt_commit
    if options.package_source_dir:
        pillar["package_source_dir"] = options.package_source_dir
    if options.package_build_dir:
        pillar["package_build_dir"] = options.package_build_dir
    if options.package_artifact_dir:
        pillar["package_artifact_dir"] = options.package_artifact_dir
    if options.pillar:
        pillar.update(dict(options.pillar))
    return salt.utils.yaml.safe_dump(
        pillar, default_flow_style=True, indent=0, width=sys.maxint
    ).rstrip()


def build_minion_target(options, vm_name):
    target = vm_name
    for grain in options.grain_target:
        target += " and G@{0}".format(grain)
    if options.grain_target:
        return '"{0}"'.format(target)
    return target


def generate_vm_name(options):
    """
    Generate a random enough vm name
    """
    if "BUILD_NUMBER" in os.environ:
        random_part = "BUILD{0:0>6}".format(os.environ.get("BUILD_NUMBER"))
    else:
        random_part = os.urandom(3).encode("hex")

    return "{0}-{1}-{2}".format(options.vm_prefix, options.platform, random_part)


def delete_vm(options):
    """
    Stop a VM
    """
    cmd = "salt-cloud -d {0} -y".format(options.delete_vm)
    print("Running CMD: {0}".format(cmd))
    sys.stdout.flush()

    proc = NonBlockingPopen(
        cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        stream_stds=True,
    )
    proc.poll_and_read_until_finish(interval=0.5)
    proc.communicate()


def echo_parseable_environment(options, parser):
    """
    Echo NAME=VAL parseable output
    """
    output = []

    if options.platform:
        name = generate_vm_name(options)
        output.extend(
            [
                "JENKINS_SALTCLOUD_VM_PLATFORM={0}".format(options.platform),
                "JENKINS_SALTCLOUD_VM_NAME={0}".format(name),
            ]
        )

    if options.provider:
        output.append("JENKINS_SALTCLOUD_VM_PROVIDER={0}".format(options.provider))

    if options.pull_request:
        # This is a Jenkins triggered Pull Request
        # We need some more data about the Pull Request available to the
        # environment
        if HAS_REQUESTS is False:
            parser.error("The python 'requests' library needs to be installed")

        headers = {}
        url = "https://api.github.com/repos/saltstack/salt/pulls/{0}".format(
            options.pull_request
        )

        github_access_token_path = os.path.join(
            os.environ.get("JENKINS_HOME", os.path.expanduser("~")), ".github_token"
        )
        if os.path.isfile(github_access_token_path):
            with salt.utils.files.fopen(github_access_token_path) as rfh:
                headers = {"Authorization": "token {0}".format(rfh.read().strip())}

        http_req = requests.get(url, headers=headers)
        if http_req.status_code != 200:
            parser.error(
                "Unable to get the pull request: {0[message]}".format(http_req.json())
            )

        pr_details = http_req.json()
        output.extend(
            [
                "SALT_PR_GIT_URL={0}".format(pr_details["head"]["repo"]["clone_url"]),
                "SALT_PR_GIT_BRANCH={0}".format(pr_details["head"]["ref"]),
                "SALT_PR_GIT_COMMIT={0}".format(pr_details["head"]["sha"]),
                "SALT_PR_GIT_BASE_BRANCH={0}".format(pr_details["base"]["ref"]),
            ]
        )

    sys.stdout.write("\n\n{0}\n\n".format("\n".join(output)))
    sys.stdout.flush()


def download_unittest_reports(options):
    print("Downloading remote unittest reports...")
    sys.stdout.flush()

    workspace = options.workspace
    xml_reports_path = os.path.join(workspace, "xml-test-reports")
    if os.path.isdir(xml_reports_path):
        shutil.rmtree(xml_reports_path)

    os.makedirs(xml_reports_path)

    cmds = (
        "salt {0} archive.tar zcvf /tmp/xml-test-reports.tar.gz '*.xml' cwd=/tmp/xml-unittests-output/",
        "salt {0} cp.push /tmp/xml-test-reports.tar.gz",
        "mv -f /var/cache/salt/master/minions/{1}/files/tmp/xml-test-reports.tar.gz {2} && "
        "tar zxvf {2}/xml-test-reports.tar.gz -C {2}/xml-test-reports && "
        "rm -f {2}/xml-test-reports.tar.gz",
    )

    vm_name = options.download_unittest_reports
    for cmd in cmds:
        cmd = cmd.format(build_minion_target(options, vm_name), vm_name, workspace)
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()

        proc = NonBlockingPopen(
            cmd,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stream_stds=True,
        )
        proc.poll_and_read_until_finish(interval=0.5)
        proc.communicate()
        if proc.returncode != 0:
            print("\nFailed to execute command. Exit code: {0}".format(proc.returncode))
        time.sleep(0.25)


def download_coverage_report(options):
    print("Downloading remote coverage report...")
    sys.stdout.flush()

    workspace = options.workspace
    vm_name = options.download_coverage_report

    if os.path.isfile(os.path.join(workspace, "coverage.xml")):
        os.unlink(os.path.join(workspace, "coverage.xml"))

    cmds = (
        "salt {0} archive.gzip /tmp/coverage.xml",
        "salt {0} cp.push /tmp/coverage.xml.gz",
        "gunzip /var/cache/salt/master/minions/{1}/files/tmp/coverage.xml.gz",
        "mv /var/cache/salt/master/minions/{1}/files/tmp/coverage.xml {2}",
    )

    for cmd in cmds:
        cmd = cmd.format(build_minion_target(options, vm_name), vm_name, workspace)
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()

        proc = NonBlockingPopen(
            cmd,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stream_stds=True,
        )
        proc.poll_and_read_until_finish(interval=0.5)
        proc.communicate()
        if proc.returncode != 0:
            print("\nFailed to execute command. Exit code: {0}".format(proc.returncode))
        time.sleep(0.25)


def download_remote_logs(options):
    print("Downloading remote logs...")
    sys.stdout.flush()

    workspace = options.workspace
    vm_name = options.download_remote_logs

    for fname in ("salt-runtests.log", "minion.log"):
        if os.path.isfile(os.path.join(workspace, fname)):
            os.unlink(os.path.join(workspace, fname))

    if not options.remote_log_path:
        options.remote_log_path = ["/tmp/salt-runtests.log", "/var/log/salt/minion"]

    cmds = []

    for remote_log in options.remote_log_path:
        cmds.extend(
            [
                "salt {{0}} archive.gzip {0}".format(remote_log),
                "salt {{0}} cp.push {0}.gz".format(remote_log),
                "gunzip /var/cache/salt/master/minions/{{1}}/files{0}.gz".format(
                    remote_log
                ),
                "mv /var/cache/salt/master/minions/{{1}}/files{0} {{2}}/{1}".format(
                    remote_log,
                    "{0}{1}".format(
                        os.path.basename(remote_log),
                        "" if remote_log.endswith(".log") else ".log",
                    ),
                ),
            ]
        )

    for cmd in cmds:
        cmd = cmd.format(build_minion_target(options, vm_name), vm_name, workspace)
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()

        proc = NonBlockingPopen(
            cmd,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stream_stds=True,
        )
        proc.poll_and_read_until_finish(interval=0.5)
        proc.communicate()
        if proc.returncode != 0:
            print("\nFailed to execute command. Exit code: {0}".format(proc.returncode))
        time.sleep(0.25)


def download_packages(options):
    print("Downloading packages...")
    sys.stdout.flush()

    workspace = options.workspace
    vm_name = options.download_packages

    for fglob in ("salt-*.rpm", "salt-*.deb", "salt-*.pkg.xz", "salt-buildpackage.log"):
        for fname in glob.glob(os.path.join(workspace, fglob)):
            if os.path.isfile(fname):
                os.unlink(fname)

    cmds = [
        (
            "salt {{0}} archive.tar czf {0}.tar.gz sources='*.*' cwd={0}".format(
                options.package_artifact_dir
            )
        ),
        "salt {{0}} cp.push {0}.tar.gz".format(options.package_artifact_dir),
        (
            "tar -C {{2}} -xzf /var/cache/salt/master/minions/{{1}}/files{0}.tar.gz".format(
                options.package_artifact_dir
            )
        ),
    ]

    for cmd in cmds:
        cmd = cmd.format(build_minion_target(options, vm_name), vm_name, workspace)
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()

        proc = NonBlockingPopen(
            cmd,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stream_stds=True,
        )
        proc.poll_and_read_until_finish(interval=0.5)
        proc.communicate()
        if proc.returncode != 0:
            print("\nFailed to execute command. Exit code: {0}".format(proc.returncode))
        time.sleep(0.25)


def run(opts):
    """
    RUN!
    """
    vm_name = os.environ.get("JENKINS_SALTCLOUD_VM_NAME", generate_vm_name(opts))

    if opts.download_remote_reports:
        if opts.test_without_coverage is False:
            opts.download_coverage_report = vm_name
        opts.download_unittest_reports = vm_name
        opts.download_packages = vm_name

    if opts.bootstrap_salt_commit is not None:
        if opts.bootstrap_salt_url is None:
            opts.bootstrap_salt_url = "https://github.com/saltstack/salt.git"
        cmd = (
            "salt-cloud -l debug"
            ' --script-args "-D -g {bootstrap_salt_url} -n git {1}"'
            " -p {provider}_{platform} {0}".format(
                vm_name,
                os.environ.get(
                    "SALT_MINION_BOOTSTRAP_RELEASE", opts.bootstrap_salt_commit
                ),
                **opts.__dict__
            )
        )
    else:
        cmd = (
            "salt-cloud -l debug"
            ' --script-args "-D -n git {1}" -p {provider}_{platform} {0}'.format(
                vm_name,
                os.environ.get(
                    "SALT_MINION_BOOTSTRAP_RELEASE", opts.bootstrap_salt_commit
                ),
                **opts.__dict__
            )
        )
    if opts.splay is not None:
        # Sleep a random number of seconds
        cloud_downtime = random.randint(0, opts.splay)
        print(
            "Sleeping random period before calling salt-cloud: {0}".format(
                cloud_downtime
            )
        )
        time.sleep(cloud_downtime)
    print("Running CMD: {0}".format(cmd))
    sys.stdout.flush()

    proc = NonBlockingPopen(
        cmd,
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        stream_stds=True,
    )
    proc.poll_and_read_until_finish(interval=0.5)
    proc.communicate()

    retcode = proc.returncode
    if retcode != 0:
        print("Failed to bootstrap VM. Exit code: {0}".format(retcode))
        sys.stdout.flush()
        if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
            delete_vm(opts)
        sys.exit(retcode)

    print("VM Bootstrapped. Exit code: {0}".format(retcode))
    sys.stdout.flush()

    # Sleep a random number of seconds
    bootstrap_downtime = random.randint(0, opts.splay)
    print(
        "Sleeping for {0} seconds to allow the minion to breathe a little".format(
            bootstrap_downtime
        )
    )
    sys.stdout.flush()
    time.sleep(bootstrap_downtime)

    if opts.bootstrap_salt_commit is not None:
        # Let's find out if the installed version matches the passed in pillar
        # information
        print("Grabbing bootstrapped minion version information ... ")
        cmd = "salt -t 100 {0} --out json test.version".format(
            build_minion_target(opts, vm_name)
        )
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()
        proc = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        )
        stdout, _ = proc.communicate()

        retcode = proc.returncode
        if retcode != 0:
            print(
                "Failed to get the bootstrapped minion version. Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

        outstr = salt.utils.stringutils.to_str(stdout).strip()
        if not outstr:
            print(
                "Failed to get the bootstrapped minion version(no output). Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

        try:
            version_info = salt.utils.json.loads(outstr)
            bootstrap_minion_version = os.environ.get(
                "SALT_MINION_BOOTSTRAP_RELEASE", opts.bootstrap_salt_commit[:7]
            )
            print("Minion reported salt version: {0}".format(version_info))
            if bootstrap_minion_version not in version_info[vm_name]:
                print("\n\nATTENTION!!!!\n")
                print(
                    "The boostrapped minion version commit does not contain the desired commit:"
                )
                print(
                    " '{0}' does not contain '{1}'".format(
                        version_info[vm_name], bootstrap_minion_version
                    )
                )
                print("\n\n")
                sys.stdout.flush()
                # if opts.clean and 'JENKINS_SALTCLOUD_VM_NAME' not in os.environ:
                #    delete_vm(opts)
                # sys.exit(retcode)
            else:
                print("matches!")
        except ValueError:
            print("Failed to load any JSON from '{0}'".format(outstr))

    if opts.cloud_only:
        # Run Cloud Provider tests preparation SLS
        cloud_provider_downtime = random.randint(3, opts.splay)
        time.sleep(cloud_provider_downtime)
        cmd = (
            'salt -t 900 {target} state.sls {cloud_prep_sls} pillar="{pillar}" '
            "--no-color".format(
                target=build_minion_target(opts, vm_name),
                cloud_prep_sls="cloud-only",
                pillar=build_pillar_data(opts),
            )
        )
    else:
        # Run standard preparation SLS
        standard_sls_downtime = random.randint(3, opts.splay)
        time.sleep(standard_sls_downtime)
        cmd = (
            'salt -t 1800 {target} state.sls {prep_sls} pillar="{pillar}" '
            "--no-color".format(
                target=build_minion_target(opts, vm_name),
                prep_sls=opts.prep_sls,
                pillar=build_pillar_data(opts),
            )
        )
    print("Running CMD: {0}".format(cmd))
    sys.stdout.flush()

    proc = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
    )
    stdout, stderr = proc.communicate()

    if stdout:
        print(salt.utils.stringutils.to_str(stdout))
    if stderr:
        print(salt.utils.stringutils.to_str(stderr))
    sys.stdout.flush()

    retcode = proc.returncode
    if retcode != 0:
        print(
            "Failed to execute the preparation SLS file. Exit code: {0}".format(retcode)
        )
        sys.stdout.flush()
        if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
            delete_vm(opts)
        sys.exit(retcode)

    if opts.cloud_only:
        cloud_provider_pillar = random.randint(3, opts.splay)
        time.sleep(cloud_provider_pillar)
        # Run Cloud Provider tests pillar preparation SLS
        cmd = (
            'salt -t 600 {target} state.sls {cloud_prep_sls} pillar="{pillar}" '
            "--no-color".format(
                target=build_minion_target(opts, vm_name),
                cloud_prep_sls="cloud-test-configs",
                pillar=build_pillar_data(opts),
            )
        )
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()

        proc = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
        )
        stdout, stderr = proc.communicate()

        if stdout:
            # DO NOT print the state return here!
            print("Cloud configuration files provisioned via pillar.")
        if stderr:
            print(salt.utils.stringutils.to_str(stderr))
        sys.stdout.flush()

        retcode = proc.returncode
        if retcode != 0:
            print(
                "Failed to execute the preparation SLS file. Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

    if opts.prep_sls_2 is not None:
        sls_2_downtime = random.randint(3, opts.splay)
        time.sleep(sls_2_downtime)

        # Run the 2nd preparation SLS
        cmd = (
            'salt -t 30 {target} state.sls {prep_sls_2} pillar="{pillar}" '
            "--no-color".format(
                prep_sls_2=opts.prep_sls_2,
                pillar=build_pillar_data(opts),
                target=build_minion_target(opts, vm_name),
            )
        )
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()

        proc = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
        )
        stdout, stderr = proc.communicate()

        if stdout:
            print(salt.utils.stringutils.to_str(stdout))
        if stderr:
            print(salt.utils.stringutils.to_str(stderr))
        sys.stdout.flush()

        retcode = proc.returncode
        if retcode != 0:
            print(
                "Failed to execute the 2nd preparation SLS file. Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

    # Run remote checks
    if opts.test_git_url is not None:
        test_git_downtime = random.randint(1, opts.splay)
        time.sleep(test_git_downtime)
        # Let's find out if the cloned repository if checked out from the
        # desired repository
        print("Grabbing the cloned repository remotes information ... ")
        cmd = "salt -t 100 {0} --out json git.remote_get /testing".format(
            build_minion_target(opts, vm_name)
        )
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()
        proc = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        )

        stdout, _ = proc.communicate()

        retcode = proc.returncode
        if retcode != 0:
            print(
                "Failed to get the cloned repository remote. Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

        if not stdout:
            print(
                "Failed to get the cloned repository remote(no output). Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

        try:
            remotes_info = salt.utils.json.loads(stdout.strip())
            if (
                remotes_info is None
                or remotes_info[vm_name] is None
                or opts.test_git_url not in remotes_info[vm_name]
            ):
                print("The cloned repository remote is not the desired one:")
                print(" '{0}' is not in {1}".format(opts.test_git_url, remotes_info))
                sys.stdout.flush()
                if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                    delete_vm(opts)
                sys.exit(retcode)
            print("matches!")
        except ValueError:
            print(
                "Failed to load any JSON from '{0}'".format(
                    salt.utils.stringutils.to_str(stdout).strip()
                )
            )

    if opts.test_git_commit is not None:
        test_git_commit_downtime = random.randint(1, opts.splay)
        time.sleep(test_git_commit_downtime)

        # Let's find out if the cloned repository is checked out at the desired
        # commit
        print("Grabbing the cloned repository commit information ... ")
        cmd = "salt -t 100 {0} --out json git.revision /testing".format(
            build_minion_target(opts, vm_name)
        )
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()
        proc = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        )
        stdout, _ = proc.communicate()
        sys.stdout.flush()

        retcode = proc.returncode
        if retcode != 0:
            print(
                "Failed to get the cloned repository revision. Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

        if not stdout:
            print(
                "Failed to get the cloned repository revision(no output). Exit code: {0}".format(
                    retcode
                )
            )
            sys.stdout.flush()
            if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                delete_vm(opts)
            sys.exit(retcode)

        try:
            revision_info = salt.utils.json.loads(stdout.strip())
            if revision_info[vm_name][7:] != opts.test_git_commit[7:]:
                print("The cloned repository commit is not the desired one:")
                print(
                    " '{0}' != '{1}'".format(
                        revision_info[vm_name][:7], opts.test_git_commit[:7]
                    )
                )
                sys.stdout.flush()
                if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
                    delete_vm(opts)
                sys.exit(retcode)
            print("matches!")
        except ValueError:
            print(
                "Failed to load any JSON from '{0}'".format(
                    salt.utils.stringutils.to_str(stdout).strip()
                )
            )

    # Run tests here
    test_begin_downtime = random.randint(3, opts.splay)
    time.sleep(test_begin_downtime)
    cmd = 'salt -t 1800 {target} state.sls {sls} pillar="{pillar}" --no-color'.format(
        sls=opts.sls,
        pillar=build_pillar_data(opts),
        target=build_minion_target(opts, vm_name),
    )
    print("Running CMD: {0}".format(cmd))
    sys.stdout.flush()

    proc = subprocess.Popen(
        cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
    )
    stdout, stderr = proc.communicate()

    outstr = salt.utils.stringutils.to_str(stdout)
    if outstr:
        print(outstr)
    if stderr:
        print(salt.utils.stringutils.to_str(stderr))
    sys.stdout.flush()

    try:
        match = re.search(r"Test Suite Exit Code: (?P<exitcode>[\d]+)", outstr)
        retcode = int(match.group("exitcode"))
    except AttributeError:
        # No regex matching
        retcode = 1
    except ValueError:
        # Not a number!?
        retcode = 1
    except TypeError:
        # No output!?
        retcode = 1
        if outstr:
            # Anything else, raise the exception
            raise

    if retcode == 0:
        # Build packages
        time.sleep(3)
        cmd = 'salt -t 1800 {target} state.sls buildpackage pillar="{pillar}" --no-color'.format(
            pillar=build_pillar_data(opts), target=build_minion_target(opts, vm_name),
        )
        print("Running CMD: {0}".format(cmd))
        sys.stdout.flush()

        proc = subprocess.Popen(
            cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
        )
        stdout, stderr = proc.communicate()

        if stdout:
            print(salt.utils.stringutils.to_str(stdout))
        if stderr:
            print(salt.utils.stringutils.to_str(stderr))
        sys.stdout.flush()

        # Grab packages and log file (or just log file if build failed)
        download_packages(opts)

    if opts.download_remote_reports:
        # Download unittest reports
        download_unittest_reports(opts)
        # Download coverage report
        if opts.test_without_coverage is False:
            download_coverage_report(opts)

    if opts.clean and "JENKINS_SALTCLOUD_VM_NAME" not in os.environ:
        delete_vm(opts)
    return retcode


def parse():
    """
    Parse the CLI options
    """
    parser = optparse.OptionParser()
    parser.add_option(
        "--vm-prefix",
        default=os.environ.get("JENKINS_VM_NAME_PREFIX", "ZJENKINS"),
        help="The bootstrapped machine name prefix",
    )
    parser.add_option(
        "-w",
        "--workspace",
        default=os.path.abspath(
            os.environ.get("WORKSPACE", os.path.dirname(os.path.dirname(__file__)))
        ),
        help="Path the execution workspace",
    )
    parser.add_option(
        "--platform",
        default=os.environ.get("JENKINS_SALTCLOUD_VM_PLATFORM", None),
        help="The target platform, choose from:\ncent6\ncent5\nubuntu12.04",
    )
    parser.add_option(
        "--provider",
        default=os.environ.get("JENKINS_SALTCLOUD_VM_PROVIDER", None),
        help="The vm provider",
    )
    parser.add_option(
        "--bootstrap-salt-url",
        default=None,
        help="The salt git repository url used to boostrap a minion",
    )
    parser.add_option(
        "--bootstrap-salt-commit",
        default=None,
        help="The salt git commit used to boostrap a minion",
    )
    parser.add_option(
        "--test-git-url", default=None, help="The testing git repository url"
    )
    parser.add_option(
        "--test-git-commit", default=None, help="The testing git commit to track"
    )
    parser.add_option(
        "--test-transport",
        default="zeromq",
        choices=("zeromq", "tcp"),
        help=(
            "Select which transport to run the integration tests with, "
            "zeromq or tcp. Default: %default"
        ),
    )
    parser.add_option(
        "--test-without-coverage",
        default=False,
        action="store_true",
        help="Do not generate coverage reports",
    )
    parser.add_option(
        "--prep-sls",
        default="git.salt",
        help="The sls file to execute to prepare the system",
    )
    parser.add_option(
        "--prep-sls-2", default=None, help="An optional 2nd system preparation SLS"
    )
    parser.add_option(
        "--sls", default="testrun-no-deps", help="The final sls file to execute"
    )
    parser.add_option(
        "--pillar",
        action="append",
        nargs=2,
        help="Pillar (key, value)s to pass to the sls file. "
        "Example: '--pillar pillar_key pillar_value'",
    )
    parser.add_option(
        "--no-clean",
        dest="clean",
        default=True,
        action="store_false",
        help="Clean up the built vm",
    )
    parser.add_option(
        "--echo-parseable-environment",
        default=False,
        action="store_true",
        help="Print a parseable KEY=VAL output",
    )
    parser.add_option("--pull-request", type=int, help="Include the PR info only")
    parser.add_option("--delete-vm", default=None, help="Delete a running VM")
    parser.add_option(
        "--download-remote-reports",
        default=False,
        action="store_true",
        help="Download remote reports when running remote 'testrun' state",
    )
    parser.add_option(
        "--download-unittest-reports",
        default=None,
        help="Download the XML unittest results",
    )
    parser.add_option(
        "--download-coverage-report",
        default=None,
        help="Download the XML coverage reports",
    )
    parser.add_option(
        "--remote-log-path",
        action="append",
        default=[],
        help="Provide additional log paths to download from remote minion",
    )
    parser.add_option(
        "--download-remote-logs",
        default=None,
        help="Download remote minion and runtests log files",
    )
    parser.add_option(
        "--grain-target",
        action="append",
        default=[],
        help="Match minions using compound matchers, the minion ID, plus the passed grain.",
    )
    parser.add_option(
        "--cloud-only",
        default=False,
        action="store_true",
        help="Run the cloud provider tests only.",
    )
    parser.add_option(
        "--build-packages",
        default=True,
        action="store_true",
        help="Run buildpackage.py to create packages off of the git build.",
    )
    # These next three options are ignored if --build-packages is False
    parser.add_option(
        "--package-source-dir",
        default="/testing",
        help="Directory where the salt source code checkout is found "
        "(default: %default)",
    )
    parser.add_option(
        "--package-build-dir",
        default="/tmp/salt-buildpackage",
        help="Build root for automated package builds (default: %default)",
    )
    parser.add_option(
        "--package-artifact-dir",
        default="/tmp/salt-packages",
        help="Location on the minion from which packages should be "
        "retrieved (default: %default)",
    )
    parser.add_option(
        "--splay",
        default="10",
        help="The number of seconds across which calls to provisioning components should be made",
    )

    options, args = parser.parse_args()

    if options.delete_vm is not None and not options.test_git_commit:
        delete_vm(options)
        parser.exit(0)

    if options.download_unittest_reports is not None and not options.test_git_commit:
        download_unittest_reports(options)
        parser.exit(0)

    if options.test_without_coverage is False:
        if options.download_coverage_report is not None and not options.test_git_commit:
            download_coverage_report(options)
            parser.exit(0)

    if options.download_remote_logs is not None and not options.test_git_commit:
        download_remote_logs(options)
        parser.exit(0)

    if not options.platform and not options.pull_request:
        parser.exit("--platform or --pull-request is required")

    if not options.provider and not options.pull_request:
        parser.exit("--provider or --pull-request is required")

    if options.echo_parseable_environment:
        echo_parseable_environment(options, parser)
        parser.exit(0)

    if not options.test_git_commit and not options.pull_request:
        parser.exit("--commit or --pull-request is required")

    return options


if __name__ == "__main__":
    exit_code = run(parse())
    print("Exit Code: {0}".format(exit_code))
    sys.exit(exit_code)



# -*- coding: utf-8 -*-

from __future__ import absolute_import, print_function, unicode_literals

import logging

import pytest
import salt.utils.path
from salt.ext import six
from salt.ext.six.moves import range
from salt.modules import mysql as mysqlmod
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest
from tests.support.mixins import SaltReturnAssertsMixin
from tests.support.unit import skipIf

log = logging.getLogger(__name__)

NO_MYSQL = False
try:
    import MySQLdb  # pylint: disable=import-error,unused-import
except Exception:  # pylint: disable=broad-except
    NO_MYSQL = True

if not salt.utils.path.which("mysqladmin"):
    NO_MYSQL = True


@skipIf(
    NO_MYSQL,
    "Please install MySQL bindings and a MySQL Server before running"
    "MySQL integration tests.",
)
@pytest.mark.windows_whitelisted
class MysqlModuleDbTest(ModuleCase, SaltReturnAssertsMixin):
    """
    Module testing database creation on a real MySQL Server.
    """

    user = "root"
    password = "poney"

    @destructiveTest
    def setUp(self):
        """
        Test presence of MySQL server, enforce a root password
        """
        super(MysqlModuleDbTest, self).setUp()
        NO_MYSQL_SERVER = True
        # now ensure we know the mysql root password
        # one of theses two at least should work
        ret1 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' flush-privileges password "'
            + self.password
            + '"',
        )
        ret2 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' --password="'
            + self.password
            + '" flush-privileges password "'
            + self.password
            + '"',
        )
        key, value = ret2.popitem()
        if value["result"]:
            NO_MYSQL_SERVER = False
        else:
            self.skipTest("No MySQL Server running, or no root access on it.")

    def _db_creation_loop(self, db_name, returning_name, test_conn=False, **kwargs):
        """
        Used in db testCase, create, check exists, check in list and removes.
        """
        ret = self.run_function("mysql.db_create", name=db_name, **kwargs)
        self.assertEqual(
            True, ret, "Problem while creating db for db name: '{0}'".format(db_name)
        )
        # test db exists
        ret = self.run_function("mysql.db_exists", name=db_name, **kwargs)
        self.assertEqual(
            True,
            ret,
            "Problem while testing db exists for db name: '{0}'".format(db_name),
        )
        # List db names to ensure db is created with the right utf8 string
        ret = self.run_function("mysql.db_list", **kwargs)
        if not isinstance(ret, list):
            raise AssertionError(
                (
                    "Unexpected query result while retrieving databases list"
                    " '{0}' for '{1}' test"
                ).format(ret, db_name)
            )
        self.assertIn(
            returning_name,
            ret,
            (
                "Problem while testing presence of db name in db lists"
                " for db name: '{0}' in list '{1}'"
            ).format(db_name, ret),
        )

        if test_conn:
            # test connections on database with root user
            ret = self.run_function(
                "mysql.query", database=db_name, query="SELECT 1", **kwargs
            )
            if not isinstance(ret, dict) or "results" not in ret:
                raise AssertionError(
                    (
                        "Unexpected result while testing connection"
                        " on database : {0}"
                    ).format(repr(db_name))
                )
            self.assertEqual([["1"]], ret["results"])

        # Now remove database
        ret = self.run_function("mysql.db_remove", name=db_name, **kwargs)
        self.assertEqual(
            True, ret, "Problem while removing db for db name: '{0}'".format(db_name)
        )

    @destructiveTest
    def test_database_creation_level1(self):
        """
        Create database, test presence, then drop db. All theses with complex names.
        """
        # name with space
        db_name = "foo 1"
        self._db_creation_loop(
            db_name=db_name,
            returning_name=db_name,
            test_conn=True,
            connection_user=self.user,
            connection_pass=self.password,
        )

        # ```````
        # create
        # also with character_set and collate only
        ret = self.run_function(
            "mysql.db_create",
            name="foo`2",
            character_set="utf8",
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        # test db exists
        ret = self.run_function(
            "mysql.db_exists",
            name="foo`2",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        # redoing the same should fail
        # even with other character sets or collations
        ret = self.run_function(
            "mysql.db_create",
            name="foo`2",
            character_set="utf8",
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(False, ret)
        # redoing the same should fail
        ret = self.run_function(
            "mysql.db_create",
            name="foo`2",
            character_set="utf8",
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(False, ret)
        # Now remove database
        ret = self.run_function(
            "mysql.db_remove",
            name="foo`2",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)

        # '''''''
        # create
        # also with character_set only
        db_name = "foo'3"
        self._db_creation_loop(
            db_name=db_name,
            returning_name=db_name,
            test_conn=True,
            character_set="utf8",
            connection_user=self.user,
            connection_pass=self.password,
        )

        # """"""""
        # also with collate only
        db_name = 'foo"4'
        self._db_creation_loop(
            db_name=db_name,
            returning_name=db_name,
            test_conn=True,
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        # fuzzy
        db_name = '<foo` --"5>'
        self._db_creation_loop(
            db_name=db_name,
            returning_name=db_name,
            test_conn=True,
            connection_user=self.user,
            connection_pass=self.password,
        )

    @destructiveTest
    def test_mysql_dbname_character_percent(self):
        """
        Play with the '%' character problems

        This character should be escaped in the form '%%' on queries, but only
        when theses queries have arguments. It is also a special character
        in LIKE SQL queries. Finally it is used to indicate query arguments.
        """
        db_name1 = "foo%1_"
        db_name2 = "foo%12"
        ret = self.run_function(
            "mysql.db_create",
            name=db_name1,
            character_set="utf8",
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        ret = self.run_function(
            "mysql.db_create",
            name=db_name2,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        ret = self.run_function(
            "mysql.db_remove",
            name=db_name1,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        ret = self.run_function(
            "mysql.db_exists",
            name=db_name1,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(False, ret)
        ret = self.run_function(
            "mysql.db_exists",
            name=db_name2,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        ret = self.run_function(
            "mysql.db_remove",
            name=db_name2,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)

    @destructiveTest
    def test_database_creation_utf8(self):
        """
        Test support of utf8 in database names
        """
        # Simple accents : using utf8 string
        db_name_unicode = "notam\xe9rican"
        # same as 'notamérican' because of file encoding
        # but ensure it on this test
        db_name_utf8 = "notam\xc3\xa9rican"
        # FIXME: MySQLdb problems on conn strings containing
        # utf-8 on user name of db name prevent conn test
        self._db_creation_loop(
            db_name=db_name_utf8,
            returning_name=db_name_utf8,
            test_conn=False,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        # test unicode entry will also return utf8 name
        self._db_creation_loop(
            db_name=db_name_unicode,
            returning_name=db_name_utf8,
            test_conn=False,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        # Using more complex unicode characters:
        db_name_unicode = "\u6a19\u6e96\u8a9e"
        # same as '標準語' because of file encoding
        # but ensure it on this test
        db_name_utf8 = "\xe6\xa8\x99\xe6\xba\x96\xe8\xaa\x9e"
        self._db_creation_loop(
            db_name=db_name_utf8,
            returning_name=db_name_utf8,
            test_conn=False,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        # test unicode entry will also return utf8 name
        self._db_creation_loop(
            db_name=db_name_unicode,
            returning_name=db_name_utf8,
            test_conn=False,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )

    @destructiveTest
    def test_database_maintenance(self):
        """
        Test maintenance operations on a created database
        """
        dbname = "foo%'-- `\"'"
        # create database
        # but first silently try to remove it
        # in case of previous tests failures
        ret = self.run_function(
            "mysql.db_remove",
            name=dbname,
            connection_user=self.user,
            connection_pass=self.password,
        )
        ret = self.run_function(
            "mysql.db_create",
            name=dbname,
            character_set="utf8",
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        # test db exists
        ret = self.run_function(
            "mysql.db_exists",
            name=dbname,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)
        # Create 3 tables
        tablenames = {
            'A%table "`1': "MYISAM",
            "B%table '`2": "InnoDB",
            "Ctable --`3": "MEMORY",
        }
        for tablename, engine in sorted(six.iteritems(tablenames)):
            # prepare queries
            create_query = (
                "CREATE TABLE {tblname} ("
                " id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,"
                " data VARCHAR(100)) ENGINE={engine};".format(
                    tblname=mysqlmod.quote_identifier(tablename), engine=engine,
                )
            )
            insert_query = "INSERT INTO {tblname} (data)" " VALUES ".format(
                tblname=mysqlmod.quote_identifier(tablename)
            )
            delete_query = "DELETE from  {tblname}" " order by rand() limit 50;".format(
                tblname=mysqlmod.quote_identifier(tablename)
            )
            for x in range(100):
                insert_query += "('foo" + six.text_type(x) + "'),"
            insert_query += "('bar');"

            # populate database
            log.info("Adding table '%s'", tablename)
            ret = self.run_function(
                "mysql.query",
                database=dbname,
                query=create_query,
                connection_user=self.user,
                connection_pass=self.password,
            )
            if not isinstance(ret, dict) or "rows affected" not in ret:
                raise AssertionError(
                    (
                        "Unexpected query result while populating test table"
                        " '{0}' : '{1}'"
                    ).format(
                        tablename, ret,
                    )
                )
            self.assertEqual(ret["rows affected"], 0)
            log.info("Populating table '%s'", tablename)
            ret = self.run_function(
                "mysql.query",
                database=dbname,
                query=insert_query,
                connection_user=self.user,
                connection_pass=self.password,
            )
            if not isinstance(ret, dict) or "rows affected" not in ret:
                raise AssertionError(
                    (
                        "Unexpected query result while populating test table"
                        " '{0}' : '{1}'"
                    ).format(
                        tablename, ret,
                    )
                )
            self.assertEqual(ret["rows affected"], 101)
            log.info("Removing some rows on table'%s'", tablename)
            ret = self.run_function(
                "mysql.query",
                database=dbname,
                query=delete_query,
                connection_user=self.user,
                connection_pass=self.password,
            )
            if not isinstance(ret, dict) or "rows affected" not in ret:
                raise AssertionError(
                    (
                        "Unexpected query result while removing rows on test table"
                        " '{0}' : '{1}'"
                    ).format(
                        tablename, ret,
                    )
                )
            self.assertEqual(ret["rows affected"], 50)
        # test check/repair/opimize on 1 table
        tablename = 'A%table "`1'
        ret = self.run_function(
            "mysql.db_check",
            name=dbname,
            table=tablename,
            connection_user=self.user,
            connection_pass=self.password,
        )
        # Note that returned result does not quote_identifier of table and db
        self.assertEqual(
            ret,
            [
                {
                    "Table": dbname + "." + tablename,
                    "Msg_text": "OK",
                    "Msg_type": "status",
                    "Op": "check",
                }
            ],
        )
        ret = self.run_function(
            "mysql.db_repair",
            name=dbname,
            table=tablename,
            connection_user=self.user,
            connection_pass=self.password,
        )
        # Note that returned result does not quote_identifier of table and db
        self.assertEqual(
            ret,
            [
                {
                    "Table": dbname + "." + tablename,
                    "Msg_text": "OK",
                    "Msg_type": "status",
                    "Op": "repair",
                }
            ],
        )
        ret = self.run_function(
            "mysql.db_optimize",
            name=dbname,
            table=tablename,
            connection_user=self.user,
            connection_pass=self.password,
        )
        # Note that returned result does not quote_identifier of table and db
        self.assertEqual(
            ret,
            [
                {
                    "Table": dbname + "." + tablename,
                    "Msg_text": "OK",
                    "Msg_type": "status",
                    "Op": "optimize",
                }
            ],
        )

        # test check/repair/opimize on all tables
        ret = self.run_function(
            "mysql.db_check",
            name=dbname,
            connection_user=self.user,
            connection_pass=self.password,
        )
        expected = []
        for tablename, engine in sorted(six.iteritems(tablenames)):
            if engine == "MEMORY":
                expected.append(
                    [
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": (
                                "The storage engine for the table doesn't"
                                " support check"
                            ),
                            "Msg_type": "note",
                            "Op": "check",
                        }
                    ]
                )
            else:
                expected.append(
                    [
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": "OK",
                            "Msg_type": "status",
                            "Op": "check",
                        }
                    ]
                )
        self.assertEqual(ret, expected)

        ret = self.run_function(
            "mysql.db_repair",
            name=dbname,
            connection_user=self.user,
            connection_pass=self.password,
        )
        expected = []
        for tablename, engine in sorted(six.iteritems(tablenames)):
            if engine == "MYISAM":
                expected.append(
                    [
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": "OK",
                            "Msg_type": "status",
                            "Op": "repair",
                        }
                    ]
                )
            else:
                expected.append(
                    [
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": (
                                "The storage engine for the table doesn't"
                                " support repair"
                            ),
                            "Msg_type": "note",
                            "Op": "repair",
                        }
                    ]
                )
        self.assertEqual(ret, expected)

        ret = self.run_function(
            "mysql.db_optimize",
            name=dbname,
            connection_user=self.user,
            connection_pass=self.password,
        )

        expected = []
        for tablename, engine in sorted(six.iteritems(tablenames)):
            if engine == "MYISAM":
                expected.append(
                    [
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": "OK",
                            "Msg_type": "status",
                            "Op": "optimize",
                        }
                    ]
                )
            elif engine == "InnoDB":
                expected.append(
                    [
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": (
                                "Table does not support optimize, "
                                "doing recreate + analyze instead"
                            ),
                            "Msg_type": "note",
                            "Op": "optimize",
                        },
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": "OK",
                            "Msg_type": "status",
                            "Op": "optimize",
                        },
                    ]
                )
            elif engine == "MEMORY":
                expected.append(
                    [
                        {
                            "Table": dbname + "." + tablename,
                            "Msg_text": (
                                "The storage engine for the table doesn't"
                                " support optimize"
                            ),
                            "Msg_type": "note",
                            "Op": "optimize",
                        }
                    ]
                )
        self.assertEqual(ret, expected)
        # Teardown, remove database
        ret = self.run_function(
            "mysql.db_remove",
            name=dbname,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(True, ret)


@skipIf(
    NO_MYSQL,
    "Please install MySQL bindings and a MySQL Server before running"
    "MySQL integration tests.",
)
@pytest.mark.windows_whitelisted
class MysqlModuleUserTest(ModuleCase, SaltReturnAssertsMixin):
    """
    User Creation and connection tests
    """

    user = "root"
    password = "poney"

    @destructiveTest
    def setUp(self):
        """
        Test presence of MySQL server, enforce a root password
        """
        super(MysqlModuleUserTest, self).setUp()
        NO_MYSQL_SERVER = True
        # now ensure we know the mysql root password
        # one of theses two at least should work
        ret1 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' flush-privileges password "'
            + self.password
            + '"',
        )
        ret2 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' --password="'
            + self.password
            + '" flush-privileges password "'
            + self.password
            + '"',
        )
        key, value = ret2.popitem()
        if value["result"]:
            NO_MYSQL_SERVER = False
        else:
            self.skipTest("No MySQL Server running, or no root access on it.")

    def _userCreationLoop(
        self,
        uname,
        host,
        password=None,
        new_password=None,
        new_password_hash=None,
        **kwargs
    ):
        """
        Perform some tests around creation of the given user
        """
        # First silently remove it, in case of
        ret = self.run_function("mysql.user_remove", user=uname, host=host, **kwargs)
        # creation
        ret = self.run_function(
            "mysql.user_create", user=uname, host=host, password=password, **kwargs
        )
        self.assertEqual(
            True,
            ret,
            ("Calling user_create on" " user '{0}' did not return True: {1}").format(
                uname, repr(ret)
            ),
        )
        # double creation failure
        ret = self.run_function(
            "mysql.user_create", user=uname, host=host, password=password, **kwargs
        )
        self.assertEqual(
            False,
            ret,
            (
                "Calling user_create a second time on"
                " user '{0}' did not return False: {1}"
            ).format(uname, repr(ret)),
        )
        # Alter password
        if new_password is not None or new_password_hash is not None:
            ret = self.run_function(
                "mysql.user_chpass",
                user=uname,
                host=host,
                password=new_password,
                password_hash=new_password_hash,
                connection_user=self.user,
                connection_pass=self.password,
                connection_charset="utf8",
                saltenv={"LC_ALL": "en_US.utf8"},
            )
            self.assertEqual(
                True,
                ret,
                (
                    "Calling user_chpass on" " user '{0}' did not return True: {1}"
                ).format(uname, repr(ret)),
            )

    def _chck_userinfo(self, user, host, check_user, check_hash):
        """
        Internal routine to check user_info returned results
        """
        ret = self.run_function(
            "mysql.user_info",
            user=user,
            host=host,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        if not isinstance(ret, dict):
            raise AssertionError(
                "Unexpected result while retrieving user_info for " "'{0}'".format(user)
            )
        self.assertEqual(ret["Host"], host)
        self.assertEqual(ret["Password"], check_hash)
        self.assertEqual(ret["User"], check_user)

    def _chk_remove_user(self, user, host, **kwargs):
        """
        Internal routine to check user_remove
        """
        ret = self.run_function("mysql.user_remove", user=user, host=host, **kwargs)
        self.assertEqual(
            True,
            ret,
            (
                "Assertion failed  while removing user" " '{0}' on host '{1}': {2}"
            ).format(user, host, repr(ret)),
        )

    @destructiveTest
    def test_user_management(self):
        """
        Test various users creation settings
        """
        # Create users with rights on this database
        # and rights on other databases
        user1 = "user '1"
        user1_pwd = "pwd`'\"1b"
        user1_pwd_hash = "*4DF33B3B12E43384677050A818327877FAB2F4BA"
        # this is : user "2'標
        user2 = "user \"2'\xe6\xa8\x99"
        user2_pwd = "user \"2'\xe6\xa8\x99b"
        user2_pwd_hash = "*3A38A7B94B024B983687BB9B44FB60B7AA38FE61"
        user3 = 'user "3;,?:@=&/'
        user3_pwd = 'user "3;,?:@=&/'
        user3_pwd_hash = "*AA3B1D4105A45D381C23A5C221C47EA349E1FD7D"
        # this is : user ":=;4標 in unicode instead of utf-8
        # if unicode char is counted as 1 char we hit the max user
        # size (16)
        user4 = 'user":;,?:@=&/4\u6a19'
        user4_utf8 = 'user":;,?:@=&/4\xe6\xa8\x99'
        user4_pwd = 'user "4;,?:@=&/'
        user4_pwd_hash = "*FC8EF8DBF27628E4E113359F8E7478D5CF3DD57C"
        user5 = 'user ``"5'
        user5_utf8 = 'user ``"5'
        # this is 標標標\
        user5_pwd = "\xe6\xa8\x99\xe6\xa8\x99\\"
        # this is password('標標\\')
        user5_pwd_hash = "*3752E65CDD8751AF8D889C62CFFC6C998B12C376"
        user6 = 'user %--"6'
        user6_utf8 = 'user %--"6'
        # this is : --'"% SIX標b
        user6_pwd_u = " --'\"% SIX\u6a19b"
        user6_pwd_utf8 = " --'\"% SIX\xe6\xa8\x99b"
        # this is password(' --\'"% SIX標b')
        user6_pwd_hash = "*90AE800593E2D407CD9E28CCAFBE42D17EEA5369"
        self._userCreationLoop(
            uname=user1,
            host="localhost",
            password="pwd`'\"1",
            new_password="pwd`'\"1b",
            connection_user=self.user,
            connection_pass=self.password,
        )
        # Now check for results
        ret = self.run_function(
            "mysql.user_exists",
            user=user1,
            host="localhost",
            password=user1_pwd,
            password_hash=None,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertEqual(
            True,
            ret,
            ("Testing final user '{0}' on host '{1}'" " existence failed").format(
                user1, "localhost"
            ),
        )

        self._userCreationLoop(
            uname=user2,
            host="localhost",
            password=None,
            # this is his name hash : user "2'標
            password_hash="*EEF6F854748ACF841226BB1C2422BEC70AE7F1FF",
            # and this is the same with a 'b' added
            new_password_hash=user2_pwd_hash,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        # user2 can connect from other places with other password
        self._userCreationLoop(
            uname=user2,
            host="10.0.0.1",
            allow_passwordless=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self._userCreationLoop(
            uname=user2,
            host="10.0.0.2",
            allow_passwordless=True,
            unix_socket=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        # Now check for results
        ret = self.run_function(
            "mysql.user_exists",
            user=user2,
            host="localhost",
            password=None,
            password_hash=user2_pwd_hash,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertEqual(
            True,
            ret,
            ("Testing final user '{0}' on host '{1}'" " failed").format(
                user2, "localhost"
            ),
        )
        ret = self.run_function(
            "mysql.user_exists",
            user=user2,
            host="10.0.0.1",
            allow_passwordless=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertEqual(
            True,
            ret,
            (
                "Testing final user '{0}' on host '{1}'" " without password failed"
            ).format(user2, "10.0.0.1"),
        )
        ret = self.run_function(
            "mysql.user_exists",
            user=user2,
            host="10.0.0.2",
            allow_passwordless=True,
            unix_socket=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertEqual(
            True,
            ret,
            (
                "Testing final user '{0}' on host '{1}'" " without password failed"
            ).format(user2, "10.0.0.2"),
        )

        # Empty password is not passwordless (or is it a bug?)
        self._userCreationLoop(
            uname=user3,
            host="localhost",
            password="",
            connection_user=self.user,
            connection_pass=self.password,
        )
        # user 3 on another host with a password
        self._userCreationLoop(
            uname=user3,
            host="%",
            password="foo",
            new_password=user3_pwd,
            connection_user=self.user,
            connection_pass=self.password,
        )
        # Now check for results
        ret = self.run_function(
            "mysql.user_exists",
            user=user3,
            host="localhost",
            password="",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(
            True,
            ret,
            (
                "Testing final user '{0}' on host '{1}'"
                " without empty password failed"
            ).format(user3, "localhost"),
        )
        ret = self.run_function(
            "mysql.user_exists",
            user=user3,
            host="%",
            password=user3_pwd,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(
            True,
            ret,
            ("Testing final user '{0}' on host '{1}'" " with password failed").format(
                user3, "%"
            ),
        )

        # check unicode name, and password > password_hash
        self._userCreationLoop(
            uname=user4,
            host="%",
            password=user4_pwd,
            # this is password('foo')
            password_hash="*F3A2A51A9B0F2BE2468926B4132313728C250DBF",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        # Now check for results
        ret = self.run_function(
            "mysql.user_exists",
            user=user4_utf8,
            host="%",
            password=user4_pwd,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertEqual(
            True,
            ret,
            (
                "Testing final user '{0}' on host '{1}'"
                " with password take from password and not password_hash"
                " failed"
            ).format(user4_utf8, "%"),
        )
        self._userCreationLoop(
            uname=user5,
            host="localhost",
            password="\xe6\xa8\x99\xe6\xa8\x99",
            new_password=user5_pwd,
            unix_socket=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        ret = self.run_function(
            "mysql.user_exists",
            user=user5_utf8,
            host="localhost",
            password=user5_pwd,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertEqual(
            True,
            ret,
            (
                "Testing final user '{0}' on host '{1}'" " with utf8 password failed"
            ).format(user5_utf8, "localhost"),
        )
        # for this one we give password in unicode and check it in utf-8
        self._userCreationLoop(
            uname=user6,
            host="10.0.0.1",
            password=" foobar",
            new_password=user6_pwd_u,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        # Now check for results
        ret = self.run_function(
            "mysql.user_exists",
            user=user6_utf8,
            host="10.0.0.1",
            password=user6_pwd_utf8,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertEqual(
            True,
            ret,
            (
                "Testing final user '{0}' on host '{1}'" " with unicode password failed"
            ).format(user6_utf8, "10.0.0.1"),
        )
        # Final result should be:
        # mysql> select Host, User, Password from user where user like 'user%';
        # +--------------------+-----------+-------------------------------+
        # | User               | Host      | Password                      |
        # +--------------------+-----------+-------------------------------+
        # | user "2'標         | 10.0.0.1  |                               |
        # | user "2'標         | 10.0.0.2  |                               |
        # | user "2'標         | localhost | *3A38A7B94B0(...)60B7AA38FE61 |
        # | user "3;,?:@=&/    | %         | *AA3B1D4105(...)47EA349E1FD7D |
        # | user "3;,?:@=&/    | localhost |                               |
        # | user %--"6         | 10.0.0.1  | *90AE800593(...)E42D17EEA5369 |
        # | user '1            | localhost | *4DF33B3B1(...)327877FAB2F4BA |
        # | user ``"5          | localhost | *3752E65CD(...)FC6C998B12C376 |
        # | user":;,?:@=&/4標  | %         | *FC8EF8DBF(...)7478D5CF3DD57C |
        # +--------------------+-----------+-------------------------------+
        self._chck_userinfo(
            user=user2, host="10.0.0.1", check_user=user2, check_hash=""
        )
        self._chck_userinfo(
            user=user2, host="10.0.0.2", check_user=user2, check_hash=""
        )
        self._chck_userinfo(
            user=user2, host="localhost", check_user=user2, check_hash=user2_pwd_hash
        )
        self._chck_userinfo(
            user=user3, host="%", check_user=user3, check_hash=user3_pwd_hash
        )
        self._chck_userinfo(
            user=user3, host="localhost", check_user=user3, check_hash=""
        )
        self._chck_userinfo(
            user=user4, host="%", check_user=user4_utf8, check_hash=user4_pwd_hash
        )
        self._chck_userinfo(
            user=user6,
            host="10.0.0.1",
            check_user=user6_utf8,
            check_hash=user6_pwd_hash,
        )
        self._chck_userinfo(
            user=user1, host="localhost", check_user=user1, check_hash=user1_pwd_hash
        )
        self._chck_userinfo(
            user=user5,
            host="localhost",
            check_user=user5_utf8,
            check_hash=user5_pwd_hash,
        )
        # check user_list function
        ret = self.run_function(
            "mysql.user_list",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertIn({"Host": "localhost", "User": user1}, ret)
        self.assertIn({"Host": "localhost", "User": user2}, ret)
        self.assertIn({"Host": "10.0.0.1", "User": user2}, ret)
        self.assertIn({"Host": "10.0.0.2", "User": user2}, ret)
        self.assertIn({"Host": "%", "User": user3}, ret)
        self.assertIn({"Host": "localhost", "User": user3}, ret)
        self.assertIn({"Host": "%", "User": user4_utf8}, ret)
        self.assertIn({"Host": "localhost", "User": user5_utf8}, ret)
        self.assertIn({"Host": "10.0.0.1", "User": user6_utf8}, ret)

        # And finally, test connections on MySQL with theses users
        ret = self.run_function(
            "mysql.query",
            database="information_schema",
            query="SELECT 1",
            connection_user=user1,
            connection_pass="pwd`'\"1b",
            connection_host="localhost",
        )
        if not isinstance(ret, dict) or "results" not in ret:
            raise AssertionError(
                (
                    "Unexpected result while testing connection" " with user '{0}': {1}"
                ).format(user1, repr(ret))
            )
        self.assertEqual([["1"]], ret["results"])

        # FIXME: still failing, but works by hand...
        # mysql --user="user \"2'標" --password="user \"2'標b" information_schema
        # Seems to be a python-mysql library problem with user names containing
        # utf8 characters
        # @see https://github.com/farcepest/MySQLdb1/issues/40
        # import urllib
        # ret = self.run_function(
        #    'mysql.query',
        #    database='information_schema',
        #    query='SELECT 1',
        #    connection_user=urllib.quote_plus(user2),
        #    connection_pass=urllib.quote_plus(user2_pwd),
        #    connection_host='localhost',
        #    connection_charset='utf8',
        #    saltenv={"LC_ALL": "en_US.utf8"}
        # )
        # if not isinstance(ret, dict) or 'results' not in ret:
        #    raise AssertionError(
        #        ('Unexpected result while testing connection'
        #        ' with user \'{0}\': {1}').format(
        #            user2,
        #            repr(ret)
        #        )
        #    )
        # self.assertEqual([['1']], ret['results'])
        ret = self.run_function(
            "mysql.query",
            database="information_schema",
            query="SELECT 1",
            connection_user=user3,
            connection_pass="",
            connection_host="localhost",
        )
        if not isinstance(ret, dict) or "results" not in ret:
            raise AssertionError(
                (
                    "Unexpected result while testing connection" " with user '{0}': {1}"
                ).format(user3, repr(ret))
            )
        self.assertEqual([["1"]], ret["results"])
        # FIXME: Failing
        # ret = self.run_function(
        #    'mysql.query',
        #    database='information_schema',
        #    query='SELECT 1',
        #    connection_user=user4_utf8,
        #    connection_pass=user4_pwd,
        #    connection_host='localhost',
        #    connection_charset='utf8',
        #    saltenv={"LC_ALL": "en_US.utf8"}
        # )
        # if not isinstance(ret, dict) or 'results' not in ret:
        #    raise AssertionError(
        #        ('Unexpected result while testing connection'
        #        ' with user \'{0}\': {1}').format(
        #            user4_utf8,
        #            repr(ret)
        #        )
        #    )
        # self.assertEqual([['1']], ret['results'])
        ret = self.run_function(
            "mysql.query",
            database="information_schema",
            query="SELECT 1",
            connection_user=user5_utf8,
            connection_pass=user5_pwd,
            connection_host="localhost",
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        if not isinstance(ret, dict) or "results" not in ret:
            raise AssertionError(
                (
                    "Unexpected result while testing connection" " with user '{0}': {1}"
                ).format(user5_utf8, repr(ret))
            )
        self.assertEqual([["1"]], ret["results"])

        # Teardown by deleting with user_remove
        self._chk_remove_user(
            user=user2,
            host="10.0.0.1",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self._chk_remove_user(
            user=user2,
            host="10.0.0.2",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self._chk_remove_user(
            user=user2,
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self._chk_remove_user(
            user=user3,
            host="%",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._chk_remove_user(
            user=user3,
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._chk_remove_user(
            user=user4,
            host="%",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self._chk_remove_user(
            user=user6,
            host="10.0.0.1",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._chk_remove_user(
            user=user1,
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._chk_remove_user(
            user=user5,
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
        )
        # Final verification of the cleanup
        ret = self.run_function(
            "mysql.user_list",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )
        self.assertNotIn({"Host": "localhost", "User": user1}, ret)
        self.assertNotIn({"Host": "localhost", "User": user2}, ret)
        self.assertNotIn({"Host": "10.0.0.1", "User": user2}, ret)
        self.assertNotIn({"Host": "10.0.0.2", "User": user2}, ret)
        self.assertNotIn({"Host": "%", "User": user3}, ret)
        self.assertNotIn({"Host": "localhost", "User": user3}, ret)
        self.assertNotIn({"Host": "%", "User": user4_utf8}, ret)
        self.assertNotIn({"Host": "localhost", "User": user5_utf8}, ret)
        self.assertNotIn({"Host": "10.0.0.1", "User": user6_utf8}, ret)


@skipIf(
    NO_MYSQL,
    "Please install MySQL bindings and a MySQL Server before running"
    "MySQL integration tests.",
)
@pytest.mark.windows_whitelisted
class MysqlModuleUserGrantTest(ModuleCase, SaltReturnAssertsMixin):
    """
    User Creation and connection tests
    """

    user = "root"
    password = "poney"
    # yep, theses are valid MySQL db names
    # very special chars are _ % and .
    testdb1 = "tes.t'\"saltdb"
    testdb2 = "t_st `(:=salt%b)"
    testdb3 = "test `(:=salteeb)"
    test_file_query_db = "test_query"
    table1 = "foo"
    table2 = "foo `'%_bar"
    users = {
        "user1": {"name": "foo", "pwd": "bar"},
        "user2": {"name": 'user ";--,?:&/\\', "pwd": '";--(),?:@=&/\\'},
        # this is : passwd 標標
        "user3": {"name": "user( @ )=foobar", "pwd": "\xe6\xa8\x99\xe6\xa8\x99"},
        # this is : user/password containing 標標
        "user4": {"name": "user \xe6\xa8\x99", "pwd": "\xe6\xa8\x99\xe6\xa8\x99"},
    }

    @destructiveTest
    def setUp(self):
        """
        Test presence of MySQL server, enforce a root password, create users
        """
        super(MysqlModuleUserGrantTest, self).setUp()
        NO_MYSQL_SERVER = True
        # now ensure we know the mysql root password
        # one of theses two at least should work
        ret1 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' flush-privileges password "'
            + self.password
            + '"',
        )
        ret2 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' --password="'
            + self.password
            + '" flush-privileges password "'
            + self.password
            + '"',
        )
        key, value = ret2.popitem()
        if value["result"]:
            NO_MYSQL_SERVER = False
        else:
            self.skipTest("No MySQL Server running, or no root access on it.")
        # Create some users and a test db
        for user, userdef in six.iteritems(self.users):
            self._userCreation(uname=userdef["name"], password=userdef["pwd"])
        self.run_function(
            "mysql.db_create",
            name=self.testdb1,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.run_function(
            "mysql.db_create",
            name=self.testdb2,
            connection_user=self.user,
            connection_pass=self.password,
        )
        create_query = (
            "CREATE TABLE {tblname} ("
            " id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,"
            " data VARCHAR(100)) ENGINE={engine};".format(
                tblname=mysqlmod.quote_identifier(self.table1), engine="MYISAM",
            )
        )
        log.info("Adding table '%s'", self.table1)
        self.run_function(
            "mysql.query",
            database=self.testdb2,
            query=create_query,
            connection_user=self.user,
            connection_pass=self.password,
        )
        create_query = (
            "CREATE TABLE {tblname} ("
            " id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,"
            " data VARCHAR(100)) ENGINE={engine};".format(
                tblname=mysqlmod.quote_identifier(self.table2), engine="MYISAM",
            )
        )
        log.info("Adding table '%s'", self.table2)
        self.run_function(
            "mysql.query",
            database=self.testdb2,
            query=create_query,
            connection_user=self.user,
            connection_pass=self.password,
        )

    @destructiveTest
    def tearDown(self):
        """
        Removes created users and db
        """
        for user, userdef in six.iteritems(self.users):
            self._userRemoval(uname=userdef["name"], password=userdef["pwd"])
        self.run_function(
            "mysql.db_remove",
            name=self.testdb1,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.run_function(
            "mysql.db_remove",
            name=self.testdb2,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.run_function(
            "mysql.db_remove",
            name=self.test_file_query_db,
            connection_user=self.user,
            connection_pass=self.password,
        )

    def _userCreation(self, uname, password=None):
        """
        Create a test user
        """
        self.run_function(
            "mysql.user_create",
            user=uname,
            host="localhost",
            password=password,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )

    def _userRemoval(self, uname, password=None):
        """
        Removes a test user
        """
        self.run_function(
            "mysql.user_remove",
            user=uname,
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
            saltenv={"LC_ALL": "en_US.utf8"},
        )

    def _addGrantRoutine(
        self, grant, user, db, grant_option=False, escape=True, **kwargs
    ):
        """
        Perform some tests around creation of the given grants
        """
        ret = self.run_function(
            "mysql.grant_add",
            grant=grant,
            database=db,
            user=user,
            grant_option=grant_option,
            escape=escape,
            **kwargs
        )
        self.assertEqual(
            True,
            ret,
            (
                "Calling grant_add on"
                " user '{0}' and grants '{1}' did not return True: {2}"
            ).format(user, grant, repr(ret)),
        )
        ret = self.run_function(
            "mysql.grant_exists",
            grant=grant,
            database=db,
            user=user,
            grant_option=grant_option,
            escape=escape,
            **kwargs
        )
        self.assertEqual(
            True,
            ret,
            (
                "Calling grant_exists on"
                " user '{0}' and grants '{1}' did not return True: {2}"
            ).format(user, grant, repr(ret)),
        )

    @destructiveTest
    def testGrants(self):
        """
        Test user grant methods
        """
        self._addGrantRoutine(
            grant="SELECT, INSERT,UPDATE, CREATE",
            user=self.users["user1"]["name"],
            db=self.testdb1 + ".*",
            grant_option=True,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._addGrantRoutine(
            grant="INSERT, SELECT",
            user=self.users["user1"]["name"],
            db=self.testdb2 + "." + self.table1,
            grant_option=True,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._addGrantRoutine(
            grant="  SELECT, UPDATE,DELETE, CREATE TEMPORARY TABLES",
            user=self.users["user2"]["name"],
            db=self.testdb1 + ".*",
            grant_option=True,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._addGrantRoutine(
            grant="select, ALTER,CREATE TEMPORARY TABLES, EXECUTE ",
            user=self.users["user3"]["name"],
            db=self.testdb1 + ".*",
            grant_option=True,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
        )
        self._addGrantRoutine(
            grant="SELECT, INSERT",
            user=self.users["user4"]["name"],
            db=self.testdb2 + "." + self.table2,
            grant_option=False,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
        )
        self._addGrantRoutine(
            grant="CREATE",
            user=self.users["user4"]["name"],
            db=self.testdb2 + ".*",
            grant_option=False,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
        )
        self._addGrantRoutine(
            grant="SELECT, INSERT",
            user=self.users["user4"]["name"],
            db=self.testdb2 + "." + self.table1,
            grant_option=False,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
        )
        # '' is valid for anonymous users
        self._addGrantRoutine(
            grant="DELETE",
            user="",
            db=self.testdb3 + ".*",
            grant_option=False,
            escape=True,
            connection_user=self.user,
            connection_pass=self.password,
        )
        # Check result for users
        ret = self.run_function(
            "mysql.user_grants",
            user=self.users["user1"]["name"],
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(
            ret,
            [
                "GRANT USAGE ON *.* TO 'foo'@'localhost'",
                (
                    "GRANT SELECT, INSERT, UPDATE, CREATE ON "
                    "`tes.t'\"saltdb`.* TO 'foo'@'localhost' WITH GRANT OPTION"
                ),
                (
                    "GRANT SELECT, INSERT ON `t_st ``(:=salt%b)`.`foo`"
                    " TO 'foo'@'localhost' WITH GRANT OPTION"
                ),
            ],
        )

        ret = self.run_function(
            "mysql.user_grants",
            user=self.users["user2"]["name"],
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(
            ret,
            [
                "GRANT USAGE ON *.* TO 'user \";--,?:&/\\'@'localhost'",
                (
                    "GRANT SELECT, UPDATE, DELETE, CREATE TEMPORARY TABLES ON `tes.t'"
                    "\"saltdb`.* TO 'user \";--,?:&/\\'@'localhost'"
                    " WITH GRANT OPTION"
                ),
            ],
        )

        ret = self.run_function(
            "mysql.user_grants",
            user=self.users["user3"]["name"],
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(
            ret,
            [
                "GRANT USAGE ON *.* TO 'user( @ )=foobar'@'localhost'",
                (
                    "GRANT SELECT, ALTER, CREATE TEMPORARY TABLES, EXECUTE ON "
                    "`tes.t'\"saltdb`.* TO 'user( @ )=foobar'@'localhost' "
                    "WITH GRANT OPTION"
                ),
            ],
        )

        ret = self.run_function(
            "mysql.user_grants",
            user=self.users["user4"]["name"],
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
            connection_charset="utf8",
        )
        self.assertEqual(
            ret,
            [
                "GRANT USAGE ON *.* TO 'user \xe6\xa8\x99'@'localhost'",
                (
                    r"GRANT CREATE ON `t\_st ``(:=salt\%b)`.* TO "
                    "'user \xe6\xa8\x99'@'localhost'"
                ),
                (
                    "GRANT SELECT, INSERT ON `t_st ``(:=salt%b)`.`foo ``'%_bar` TO "
                    "'user \xe6\xa8\x99'@'localhost'"
                ),
                (
                    "GRANT SELECT, INSERT ON `t_st ``(:=salt%b)`.`foo` TO "
                    "'user \xe6\xa8\x99'@'localhost'"
                ),
            ],
        )

        ret = self.run_function(
            "mysql.user_grants",
            user="",
            host="localhost",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertEqual(
            ret,
            [
                "GRANT USAGE ON *.* TO ''@'localhost'",
                "GRANT DELETE ON `test ``(:=salteeb)`.* TO ''@'localhost'",
            ],
        )


@skipIf(
    NO_MYSQL,
    "Please install MySQL bindings and a MySQL Server before running"
    "MySQL integration tests.",
)
@pytest.mark.windows_whitelisted
class MysqlModuleFileQueryTest(ModuleCase, SaltReturnAssertsMixin):
    """
    Test file query module
    """

    user = "root"
    password = "poney"
    testdb = "test_file_query"

    @destructiveTest
    def setUp(self):
        """
        Test presence of MySQL server, enforce a root password, create users
        """
        super(MysqlModuleFileQueryTest, self).setUp()
        NO_MYSQL_SERVER = True
        # now ensure we know the mysql root password
        # one of theses two at least should work
        ret1 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' flush-privileges password "'
            + self.password
            + '"',
        )
        ret2 = self.run_state(
            "cmd.run",
            name='mysqladmin --host="localhost" -u '
            + self.user
            + ' --password="'
            + self.password
            + '" flush-privileges password "'
            + self.password
            + '"',
        )
        key, value = ret2.popitem()
        if value["result"]:
            NO_MYSQL_SERVER = False
        else:
            self.skipTest("No MySQL Server running, or no root access on it.")
        # Create some users and a test db
        self.run_function(
            "mysql.db_create",
            name=self.testdb,
            connection_user=self.user,
            connection_pass=self.password,
            connection_db="mysql",
        )

    @destructiveTest
    def tearDown(self):
        """
        Removes created users and db
        """
        self.run_function(
            "mysql.db_remove",
            name=self.testdb,
            connection_user=self.user,
            connection_pass=self.password,
            connection_db="mysql",
        )

    @destructiveTest
    def test_update_file_query(self):
        """
        Test query without any output
        """
        ret = self.run_function(
            "mysql.file_query",
            database=self.testdb,
            file_name="salt://mysql/update_query.sql",
            character_set="utf8",
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        self.assertTrue("query time" in ret)
        ret.pop("query time")
        self.assertEqual(ret, {"rows affected": 2})

    @destructiveTest
    def test_select_file_query(self):
        """
        Test query with table output
        """
        ret = self.run_function(
            "mysql.file_query",
            database=self.testdb,
            file_name="salt://mysql/select_query.sql",
            character_set="utf8",
            collate="utf8_general_ci",
            connection_user=self.user,
            connection_pass=self.password,
        )
        expected = {
            "rows affected": 5,
            "rows returned": 4,
            "results": [[["2"], ["3"], ["4"], ["5"]]],
            "columns": [["a"]],
        }
        self.assertTrue("query time" in ret)
        ret.pop("query time")
        self.assertEqual(ret, expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.shell.call
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
"""
from __future__ import absolute_import

import logging
import os
import re
import shutil
import sys

import pytest
import salt.utils.files
import salt.utils.json
import salt.utils.platform
import salt.utils.yaml
from salt.ext import six
from tests.integration.utils import testprogram
from tests.support.case import ShellCase
from tests.support.helpers import change_cwd, flaky, slowTest, with_tempfile
from tests.support.mixins import ShellCaseCommonTestsMixin
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@pytest.mark.windows_whitelisted
class CallTest(ShellCase, testprogram.TestProgramCase, ShellCaseCommonTestsMixin):

    _call_binary_ = "salt-call"

    @slowTest
    def test_default_output(self):
        out = self.run_call("-l quiet test.fib 3")

        expect = ["local:", "    - 2"]
        self.assertEqual(expect, out[:-1])

    @slowTest
    def test_text_output(self):
        out = self.run_call("-l quiet --out txt test.fib 3")

        expect = ["local: (2"]

        self.assertEqual("".join(expect), "".join(out).rsplit(",", 1)[0])

    @slowTest
    def test_json_out_indent(self):
        out = self.run_call("test.ping -l quiet --out=json --out-indent=-1")
        self.assertIn('"local": true', "".join(out))

        out = self.run_call("test.ping -l quiet --out=json --out-indent=0")
        self.assertIn('"local": true', "".join(out))

        out = self.run_call("test.ping -l quiet --out=json --out-indent=1")
        self.assertIn('"local": true', "".join(out))

    @slowTest
    def test_local_sls_call(self):
        fileroot = os.path.join(RUNTIME_VARS.FILES, "file", "base")
        out = self.run_call(
            "--file-root {0} state.sls saltcalllocal".format(fileroot), local=True
        )
        self.assertIn("Name: test.echo", "".join(out))
        self.assertIn("Result: True", "".join(out))
        self.assertIn("hello", "".join(out))
        self.assertIn("Succeeded: 1", "".join(out))

    @with_tempfile()
    @slowTest
    def test_local_salt_call(self, name):
        """
        This tests to make sure that salt-call does not execute the
        function twice, see https://github.com/saltstack/salt/pull/49552
        """

        def _run_call(cmd):
            cmd = "--out=json " + cmd
            return salt.utils.json.loads("".join(self.run_call(cmd, local=True)))[
                "local"
            ]

        ret = _run_call('state.single file.append name={0} text="foo"'.format(name))
        ret = ret[next(iter(ret))]

        # Make sure we made changes
        assert ret["changes"]

        # 2nd sanity check: make sure that "foo" only exists once in the file
        with salt.utils.files.fopen(name) as fp_:
            contents = fp_.read()
        assert contents.count("foo") == 1, contents

    @skipIf(
        salt.utils.platform.is_windows() or salt.utils.platform.is_darwin(),
        "This test requires a supported master",
    )
    @slowTest
    def test_user_delete_kw_output(self):
        ret = self.run_call("-l quiet -d user.delete")
        assert "salt '*' user.delete name remove=True force=True" in "".join(ret)

    @slowTest
    def test_salt_documentation_too_many_arguments(self):
        """
        Test to see if passing additional arguments shows an error
        """
        data = self.run_call("-d virtualenv.create /tmp/ve", catch_stderr=True)
        self.assertIn(
            "You can only get documentation for one method at one time",
            "\n".join(data[1]),
        )

    @slowTest
    def test_issue_6973_state_highstate_exit_code(self):
        """
        If there is no tops/master_tops or state file matches
        for this minion, salt-call should exit non-zero if invoked with
        option --retcode-passthrough
        """
        src = os.path.join(RUNTIME_VARS.BASE_FILES, "top.sls")
        dst = os.path.join(RUNTIME_VARS.BASE_FILES, "top.sls.bak")
        shutil.move(src, dst)
        expected_comment = "No states found for this minion"
        try:
            stdout, retcode = self.run_call(
                "-l quiet --retcode-passthrough state.highstate", with_retcode=True
            )
        finally:
            shutil.move(dst, src)
        self.assertIn(expected_comment, "".join(stdout))
        self.assertNotEqual(0, retcode)

    @skipIf(sys.platform.startswith("win"), "This test does not apply on Win")
    @skipIf(True, "to be re-enabled when #23623 is merged")
    def test_return(self):
        self.run_call('cmd.run "echo returnTOmaster"')
        jobs = [a for a in self.run_run("jobs.list_jobs")]

        self.assertTrue(True in ["returnTOmaster" in j for j in jobs])
        # lookback jid
        first_match = [(i, j) for i, j in enumerate(jobs) if "returnTOmaster" in j][0]
        jid, idx = None, first_match[0]
        while idx > 0:
            jid = re.match("([0-9]+):", jobs[idx])
            if jid:
                jid = jid.group(1)
                break
            idx -= 1
        assert idx > 0
        assert jid
        master_out = [a for a in self.run_run("jobs.lookup_jid {0}".format(jid))]
        self.assertTrue(True in ["returnTOmaster" in a for a in master_out])

    @skipIf(salt.utils.platform.is_windows(), "Skip on Windows")
    @slowTest
    def test_syslog_file_not_found(self):
        """
        test when log_file is set to a syslog file that does not exist
        """
        config_dir = os.path.join(RUNTIME_VARS.TMP, "log_file_incorrect")
        if not os.path.isdir(config_dir):
            os.makedirs(config_dir)

        with change_cwd(config_dir):
            with salt.utils.files.fopen(
                self.get_config_file_path("minion"), "r"
            ) as fh_:
                minion_config = salt.utils.yaml.load(fh_.read())
                minion_config["log_file"] = "file:///dev/doesnotexist"
                with salt.utils.files.fopen(
                    os.path.join(config_dir, "minion"), "w"
                ) as fh_:
                    fh_.write(
                        salt.utils.yaml.dump(minion_config, default_flow_style=False)
                    )
            ret = self.run_script(
                "salt-call",
                '--config-dir {0} cmd.run "echo foo"'.format(config_dir),
                timeout=120,
                catch_stderr=True,
                with_retcode=True,
            )
            try:
                if sys.version_info >= (3, 5, 4):
                    self.assertIn("local:", ret[0])
                    self.assertIn(
                        "[WARNING ] The log_file does not exist. Logging not setup correctly or syslog service not started.",
                        ret[1],
                    )
                    self.assertEqual(ret[2], 0)
                else:
                    self.assertIn(
                        "Failed to setup the Syslog logging handler", "\n".join(ret[1])
                    )
                    self.assertEqual(ret[2], 2)
            finally:
                if os.path.isdir(config_dir):
                    shutil.rmtree(config_dir)

    @skipIf(True, "This test is unreliable. Need to investigate why more deeply.")
    @flaky
    def test_issue_15074_output_file_append(self):
        output_file_append = os.path.join(RUNTIME_VARS.TMP, "issue-15074")
        try:
            # Let's create an initial output file with some data
            _ = self.run_script(
                "salt-call",
                "-c {0} --output-file={1} test.versions".format(
                    RUNTIME_VARS.TMP_MINION_CONF_DIR, output_file_append
                ),
                catch_stderr=True,
                with_retcode=True,
            )

            with salt.utils.files.fopen(output_file_append) as ofa:
                output = ofa.read()

            self.run_script(
                "salt-call",
                "-c {0} --output-file={1} --output-file-append test.versions".format(
                    self.config_dir, output_file_append
                ),
                catch_stderr=True,
                with_retcode=True,
            )
            with salt.utils.files.fopen(output_file_append) as ofa:
                self.assertEqual(ofa.read(), output + output)
        finally:
            if os.path.exists(output_file_append):
                os.unlink(output_file_append)

    @skipIf(True, "This test is unreliable. Need to investigate why more deeply.")
    @flaky
    def test_issue_14979_output_file_permissions(self):
        output_file = os.path.join(RUNTIME_VARS.TMP, "issue-14979")
        with salt.utils.files.set_umask(0o077):
            try:
                # Let's create an initial output file with some data
                self.run_script(
                    "salt-call",
                    "-c {0} --output-file={1} -l trace -g".format(
                        RUNTIME_VARS.TMP_MINION_CONF_DIR, output_file
                    ),
                    catch_stderr=True,
                    with_retcode=True,
                )
                try:
                    stat1 = os.stat(output_file)
                except OSError:
                    self.fail("Failed to generate output file, see log for details")

                # Let's change umask
                os.umask(0o777)  # pylint: disable=blacklisted-function

                self.run_script(
                    "salt-call",
                    "-c {0} --output-file={1} --output-file-append -g".format(
                        RUNTIME_VARS.TMP_MINION_CONF_DIR, output_file
                    ),
                    catch_stderr=True,
                    with_retcode=True,
                )
                try:
                    stat2 = os.stat(output_file)
                except OSError:
                    self.fail("Failed to generate output file, see log for details")
                self.assertEqual(stat1.st_mode, stat2.st_mode)
                # Data was appeneded to file
                self.assertTrue(stat1.st_size < stat2.st_size)

                # Let's remove the output file
                os.unlink(output_file)

                # Not appending data
                self.run_script(
                    "salt-call",
                    "-c {0} --output-file={1} -g".format(
                        RUNTIME_VARS.TMP_MINION_CONF_DIR, output_file
                    ),
                    catch_stderr=True,
                    with_retcode=True,
                )
                try:
                    stat3 = os.stat(output_file)
                except OSError:
                    self.fail("Failed to generate output file, see log for details")
                # Mode must have changed since we're creating a new log file
                self.assertNotEqual(stat1.st_mode, stat3.st_mode)
            finally:
                if os.path.exists(output_file):
                    os.unlink(output_file)

    @skipIf(sys.platform.startswith("win"), "This test does not apply on Win")
    @slowTest
    def test_42116_cli_pillar_override(self):
        ret = self.run_call(
            "state.apply issue-42116-cli-pillar-override "
            'pillar=\'{"myhost": "localhost"}\''
        )
        for line in ret:
            line = line.lstrip()
            if line == 'Comment: Command "ping -c 2 localhost" run':
                # Successful test
                break
        else:
            log.debug("salt-call output:\n\n%s", "\n".join(ret))
            self.fail("CLI pillar override not found in pillar data")

    @slowTest
    def test_pillar_items_masterless(self):
        """
        Test to ensure we get expected output
        from pillar.items with salt-call
        """
        get_items = self.run_call("pillar.items", local=True)
        exp_out = [
            "        - Lancelot",
            "        - Galahad",
            "        - Bedevere",
            "    monty:",
            "        python",
        ]
        for out in exp_out:
            self.assertIn(out, get_items)

    def tearDown(self):
        """
        Teardown method to remove installed packages
        """
        user = ""
        user_info = self.run_call(" grains.get username", local=True)
        if (
            user_info
            and isinstance(user_info, (list, tuple))
            and isinstance(user_info[-1], six.string_types)
        ):
            user = user_info[-1].strip()
        super(CallTest, self).tearDown()

    @slowTest
    def test_exit_status_unknown_argument(self):
        """
        Ensure correct exit status when an unknown argument is passed to salt-call.
        """

        call = testprogram.TestProgramSaltCall(
            name="unknown_argument", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        call.setup()
        stdout, stderr, status = call.run(
            args=["--unknown-argument"], catch_stderr=True, with_retcode=True,
        )
        self.assert_exit_status(
            status, "EX_USAGE", message="unknown argument", stdout=stdout, stderr=stderr
        )

    @slowTest
    def test_masterless_highstate(self):
        """
        test state.highstate in masterless mode
        """
        ret = self.run_call("state.highstate", local=True)

        destpath = os.path.join(RUNTIME_VARS.TMP, "testfile")
        exp_out = [
            "    Function: file.managed",
            "      Result: True",
            "          ID: {0}".format(destpath),
        ]

        for out in exp_out:
            self.assertIn(out, ret)

        self.assertTrue(os.path.exists(destpath))

    @slowTest
    def test_exit_status_correct_usage(self):
        """
        Ensure correct exit status when salt-call starts correctly.
        """

        call = testprogram.TestProgramSaltCall(
            name="correct_usage", parent_dir=self._test_dir,
        )
        # Call setup here to ensure config and script exist
        call.setup()
        stdout, stderr, status = call.run(
            args=["--local", "test.true"], catch_stderr=True, with_retcode=True,
        )
        self.assert_exit_status(
            status, "EX_OK", message="correct usage", stdout=stdout, stderr=stderr
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_asg as boto_asg

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class BotoAsgTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.boto_asg
    """

    # 'present' function tests: 1

    def setup_loader_modules(self):
        return {boto_asg: {}}

    maxSize = None

    def test_present(self):
        """
        Test to ensure the autoscale group exists.
        """
        name = "myasg"
        launch_config_name = "mylc"
        availability_zones = ["us-east-1a", "us-east-1b"]
        min_size = 1
        max_size = 1

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[False, {"min_size": 2}, [""]])
        with patch.dict(boto_asg.__salt__, {"boto_asg.get_config": mock}):
            with patch.dict(boto_asg.__opts__, {"test": True}):
                comt = "Autoscale group set to be created."
                ret.update({"comment": comt})
                with patch.dict(
                    boto_asg.__salt__, {"config.option": MagicMock(return_value={})}
                ):
                    self.assertDictEqual(
                        boto_asg.present(
                            name,
                            launch_config_name,
                            availability_zones,
                            min_size,
                            max_size,
                        ),
                        ret,
                    )

                def magic_side_effect(value):
                    if isinstance(value, int):
                        if value == 1:
                            return 4
                        return value
                    return ""

                comt = "Autoscale group set to be updated."
                ret.update({"comment": comt, "result": None})
                ret.update(
                    {"changes": {"new": {"min_size": 4}, "old": {"min_size": 2}}}
                )
                utils_ordered_mock = MagicMock(side_effect=magic_side_effect)
                with patch.dict(
                    boto_asg.__salt__, {"config.option": MagicMock(return_value={})}
                ):
                    with patch.dict(
                        boto_asg.__utils__, {"boto3.ordered": utils_ordered_mock}
                    ):
                        call_ret = boto_asg.present(
                            name,
                            launch_config_name,
                            availability_zones,
                            min_size,
                            max_size,
                        )
                        self.assertDictEqual(call_ret, ret)

                with patch.dict(
                    boto_asg.__salt__, {"config.option": MagicMock(return_value={})}
                ):
                    with patch.dict(
                        boto_asg.__utils__,
                        {"boto3.ordered": MagicMock(return_value="")},
                    ):
                        comt = "Autoscale group present. "
                        ret.update({"comment": comt, "result": True})
                        ret.update({"changes": {}})
                        self.assertDictEqual(
                            boto_asg.present(
                                name,
                                launch_config_name,
                                availability_zones,
                                min_size,
                                max_size,
                            ),
                            ret,
                        )

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure the named autoscale group is deleted.
        """
        name = "myasg"

        ret = {"name": name, "result": None, "changes": {}, "comment": ""}

        mock = MagicMock(side_effect=[True, False])
        with patch.dict(boto_asg.__salt__, {"boto_asg.get_config": mock}):
            with patch.dict(boto_asg.__opts__, {"test": True}):
                comt = "Autoscale group set to be deleted."
                ret.update({"comment": comt})
                self.assertDictEqual(boto_asg.absent(name), ret)

                comt = "Autoscale group does not exist."
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(boto_asg.absent(name), ret)



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.mac_assistive as assistive

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class AssistiveTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {assistive: {}}

    def test_installed(self):
        """
            Test installing a bundle ID as being allowed to run with assistive access
        """
        expected = {
            "changes": {},
            "comment": "Installed com.apple.Chess into the assistive access panel",
            "name": "com.apple.Chess",
            "result": True,
        }

        installed_mock = MagicMock(return_value=False)
        install_mock = MagicMock()

        with patch.dict(
            assistive.__salt__,
            {"assistive.installed": installed_mock, "assistive.install": install_mock},
        ):
            out = assistive.installed("com.apple.Chess")
            installed_mock.assert_called_once_with("com.apple.Chess")
            install_mock.assert_called_once_with("com.apple.Chess", True)
            self.assertEqual(out, expected)

    def test_installed_not_enabled(self):
        """
            Test installing a bundle ID as being allowed to run with assistive access
        """
        expected = {
            "changes": {},
            "comment": "Updated enable to True",
            "name": "com.apple.Chess",
            "result": True,
        }

        installed_mock = MagicMock(return_value=True)
        install_mock = MagicMock()
        enabled_mock = MagicMock(return_value=False)
        enable_mock = MagicMock()

        with patch.dict(
            assistive.__salt__,
            {
                "assistive.installed": installed_mock,
                "assistive.install": install_mock,
                "assistive.enabled": enabled_mock,
                "assistive.enable": enable_mock,
            },
        ):
            out = assistive.installed("com.apple.Chess")
            enabled_mock.assert_called_once_with("com.apple.Chess")
            enable_mock.assert_called_once_with("com.apple.Chess", True)
            assert not install_mock.called
            self.assertEqual(out, expected)

    def test_installed_enabled(self):
        """
            Test enabling an already enabled bundle ID
        """
        expected = {
            "changes": {},
            "comment": "Already in the correct state",
            "name": "com.apple.Chess",
            "result": True,
        }

        installed_mock = MagicMock(return_value=True)
        install_mock = MagicMock()
        enabled_mock = MagicMock(return_value=True)
        enable_mock = MagicMock()

        with patch.dict(
            assistive.__salt__,
            {
                "assistive.installed": installed_mock,
                "assistive.install": install_mock,
                "assistive.enabled": enabled_mock,
                "assistive.enable": enable_mock,
            },
        ):
            out = assistive.installed("com.apple.Chess")
            enabled_mock.assert_called_once_with("com.apple.Chess")
            assert not enable_mock.called
            assert not install_mock.called
            self.assertEqual(out, expected)

    def test_installed_not_disabled(self):
        """
            Test disabling an enabled and installed bundle ID
        """
        expected = {
            "changes": {},
            "comment": "Updated enable to False",
            "name": "com.apple.Chess",
            "result": True,
        }

        installed_mock = MagicMock(return_value=True)
        install_mock = MagicMock()
        enabled_mock = MagicMock(return_value=True)
        enable_mock = MagicMock()

        with patch.dict(
            assistive.__salt__,
            {
                "assistive.installed": installed_mock,
                "assistive.install": install_mock,
                "assistive.enabled": enabled_mock,
                "assistive.enable": enable_mock,
            },
        ):
            out = assistive.installed("com.apple.Chess", False)
            enabled_mock.assert_called_once_with("com.apple.Chess")
            enable_mock.assert_called_once_with("com.apple.Chess", False)
            assert not install_mock.called
            self.assertEqual(out, expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.states.test as test

# Import Salt Libs
from salt.exceptions import SaltInvocationError
from salt.ext import six
from salt.utils.odict import OrderedDict

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class TestTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the test state
    """

    def setup_loader_modules(self):
        return {test: {"__low__": {"__reqs__": {"watch": ""}}}}

    def test_succeed_without_changes(self):
        """
            Test to returns successful.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        with patch.dict(test.__opts__, {"test": False}):
            ret.update({"comment": "Success!"})
            self.assertDictEqual(test.succeed_without_changes("salt"), ret)

        with patch.dict(test.__opts__, {"test": False}):
            ret.update({"comment": "A success comment!"})
            self.assertDictEqual(
                test.succeed_without_changes("salt", comment="A success comment!"), ret
            )

    def test_fail_without_changes(self):
        """
            Test to returns failure.
        """
        ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
        with patch.dict(test.__opts__, {"test": False}):
            ret.update({"comment": "Failure!"})
            self.assertDictEqual(test.fail_without_changes("salt"), ret)

        with patch.dict(test.__opts__, {"test": False}):
            ret.update({"comment": "A failure comment!"})
            self.assertDictEqual(
                test.fail_without_changes("salt", comment="A failure comment!"), ret
            )

    def test_succeed_with_changes(self):
        """
            Test to returns successful and changes is not empty
        """
        ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
        with patch.dict(test.__opts__, {"test": False}):
            ret.update(
                {
                    "changes": {
                        "testing": {
                            "new": "Something pretended" " to change",
                            "old": "Unchanged",
                        }
                    },
                    "comment": "Success!",
                    "result": True,
                }
            )
            self.assertDictEqual(test.succeed_with_changes("salt"), ret)

        with patch.dict(test.__opts__, {"test": False}):
            ret.update(
                {
                    "changes": {
                        "testing": {
                            "new": "Something pretended" " to change",
                            "old": "Unchanged",
                        }
                    },
                    "comment": "A success comment!",
                    "result": True,
                }
            )
            self.assertDictEqual(
                test.succeed_with_changes("salt", comment="A success comment!"), ret
            )

    def test_fail_with_changes(self):
        """
            Test to returns failure and changes is not empty.
        """
        ret = {"name": "salt", "changes": {}, "result": False, "comment": ""}
        with patch.dict(test.__opts__, {"test": False}):
            ret.update(
                {
                    "changes": {
                        "testing": {
                            "new": "Something pretended" " to change",
                            "old": "Unchanged",
                        }
                    },
                    "comment": "Success!",
                    "result": True,
                }
            )
            self.assertDictEqual(test.succeed_with_changes("salt"), ret)

        with patch.dict(test.__opts__, {"test": False}):
            ret.update(
                {
                    "changes": {
                        "testing": {
                            "new": "Something pretended" " to change",
                            "old": "Unchanged",
                        }
                    },
                    "comment": "A failure comment!",
                    "result": True,
                }
            )
            self.assertDictEqual(
                test.succeed_with_changes("salt", comment="A failure comment!"), ret
            )

    def test_configurable_test_state(self):
        """
        Test test.configurable_test_state with and without comment
        """
        # Configure mock parameters
        mock_name = "cheese_shop"
        mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
        mock_changes = {
            "testing": {"old": "Unchanged", "new": "Something pretended to change"}
        }

        # Test default state with comment
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": True,
                "comment": "",
            }
            ret = test.configurable_test_state(mock_name)
            self.assertDictEqual(ret, mock_ret)

        # Test default state without comment
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": True,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(mock_name, comment=mock_comment)
            self.assertDictEqual(ret, mock_ret)

    def test_configurable_test_state_changes(self):
        """
        Test test.configurable_test_state with permutations of changes and with
        comment
        """
        # Configure mock parameters
        mock_name = "cheese_shop"
        mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
        mock_changes = {
            "testing": {"old": "Unchanged", "new": "Something pretended to change"}
        }

        # Test changes=Random and comment
        with patch.dict(test.__opts__, {"test": False}):
            ret = test.configurable_test_state(
                mock_name, changes="Random", comment=mock_comment
            )
            self.assertEqual(ret["name"], mock_name)
            self.assertIn(ret["changes"], [mock_changes, {}])
            self.assertEqual(ret["result"], True)
            self.assertEqual(ret["comment"], mock_comment)

        # Test changes=True and comment
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": True,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(
                mock_name, changes=True, comment=mock_comment
            )
            self.assertDictEqual(ret, mock_ret)

        # Test changes=False and comment
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": {},
                "result": True,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(
                mock_name, changes=False, comment=mock_comment
            )
            self.assertDictEqual(ret, mock_ret)

        # Test changes=Cheese
        with patch.dict(test.__opts__, {"test": False}):
            self.assertRaises(
                SaltInvocationError,
                test.configurable_test_state,
                mock_name,
                changes="Cheese",
            )

    def test_configurable_test_state_result(self):
        """
        Test test.configurable_test_state with permutations of result and with
        comment
        """
        # Configure mock parameters
        mock_name = "cheese_shop"
        mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
        mock_changes = {
            "testing": {"old": "Unchanged", "new": "Something pretended to change"}
        }

        # Test result=Random and comment
        with patch.dict(test.__opts__, {"test": False}):
            ret = test.configurable_test_state(
                mock_name, result="Random", comment=mock_comment
            )
            self.assertEqual(ret["name"], mock_name)
            self.assertEqual(ret["changes"], mock_changes)
            self.assertIn(ret["result"], [True, False])
            self.assertEqual(ret["comment"], mock_comment)

        # Test result=True and comment
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": True,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(
                mock_name, result=True, comment=mock_comment
            )
            self.assertDictEqual(ret, mock_ret)

        # Test result=False and comment
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": False,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(
                mock_name, result=False, comment=mock_comment
            )
            self.assertDictEqual(ret, mock_ret)

        # Test result=Cheese
        with patch.dict(test.__opts__, {"test": False}):
            self.assertRaises(
                SaltInvocationError,
                test.configurable_test_state,
                mock_name,
                result="Cheese",
            )

    def test_configurable_test_state_warnings(self):
        """
        Test test.configurable_test_state with and without warnings
        """
        # Configure mock parameters
        mock_name = "cheese_shop"
        mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
        mock_warning = "Today the van broke down."
        mock_warning_list = [mock_warning, "Oooooooooohhh........!"]
        mock_changes = {
            "testing": {"old": "Unchanged", "new": "Something pretended to change"}
        }

        # Test default state without warnings
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": True,
                "comment": "",
            }
            ret = test.configurable_test_state(mock_name)
            self.assertDictEqual(ret, mock_ret)

        # Test default state with warnings (string)
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": True,
                "comment": "",
                "warnings": mock_warning_list,
            }
            ret = test.configurable_test_state(mock_name, warnings=mock_warning_list)
            self.assertDictEqual(ret, mock_ret)

        # Test default state with warnings (list)
        with patch.dict(test.__opts__, {"test": False}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": True,
                "comment": "",
                "warnings": ["Today the van broke down."],
            }
            ret = test.configurable_test_state(mock_name, warnings=mock_warning)

            self.assertDictEqual(ret, mock_ret)

    def test_configurable_test_state_test(self):
        """
        Test test.configurable_test_state with test=True with and without
        comment
        """
        # Configure mock parameters
        mock_name = "cheese_shop"
        mock_comment = "I'm afraid we're fresh out of Red Leicester sir."
        mock_changes = {
            "testing": {"old": "Unchanged", "new": "Something pretended to change"}
        }

        # Test test=True without comment
        with patch.dict(test.__opts__, {"test": True}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": None,
                "comment": "This is a test",
            }
            ret = test.configurable_test_state(mock_name)
            self.assertDictEqual(ret, mock_ret)

        # Test test=True with comment
        with patch.dict(test.__opts__, {"test": True}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": None,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(mock_name, comment=mock_comment)
            self.assertDictEqual(ret, mock_ret)

        # Test test=True and changes=True with comment
        with patch.dict(test.__opts__, {"test": True}):
            mock_ret = {
                "name": mock_name,
                "changes": mock_changes,
                "result": None,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(
                mock_name, changes=True, comment=mock_comment
            )
            self.assertDictEqual(ret, mock_ret)

        # Test test=True and changes=False with comment
        with patch.dict(test.__opts__, {"test": True}):
            mock_ret = {
                "name": mock_name,
                "changes": {},
                "result": True,
                "comment": mock_comment,
            }
            ret = test.configurable_test_state(
                mock_name, changes=False, comment=mock_comment
            )
            self.assertDictEqual(ret, mock_ret)

    def test_mod_watch(self):
        """
            Test to call this function via a watch statement
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        ret.update(
            {
                "changes": {"Requisites with changes": []},
                "comment": "Watch statement fired.",
            }
        )
        self.assertDictEqual(test.mod_watch("salt"), ret)

    def test_check_pillar_present(self):
        """
            Test to ensure the check_pillar function
            works properly with the 'present' keyword in
            the absence of a 'type' keyword.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        pillar_return = "I am a pillar."
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertEqual(test.check_pillar("salt", present="my_pillar"), ret)

    def test_check_pillar_string(self):
        """
            Test to ensure the check_pillar function
            works properly with the 'key_type' checks,
            using the string key_type.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        pillar_return = "I am a pillar."
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertEqual(test.check_pillar("salt", string="my_pillar"), ret)
        # With unicode (py2) or str (py3) strings
        pillar_return = six.text_type("I am a pillar.")
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertEqual(test.check_pillar("salt", string="my_pillar"), ret)
        # With a dict
        pillar_return = {"this": "dictionary"}
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(test.check_pillar("salt", string="my_pillar")["result"])
        # With a list
        pillar_return = ["I am a pillar."]
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(test.check_pillar("salt", string="my_pillar")["result"])
        # With a boolean
        pillar_return = True
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(test.check_pillar("salt", string="my_pillar")["result"])
        # With an int
        pillar_return = 1
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(test.check_pillar("salt", string="my_pillar")["result"])

    def test_check_pillar_dictionary(self):
        """
            Test to ensure the check_pillar function
            works properly with the 'key_type' checks,
            using the dictionary key_type.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        pillar_return = {"this": "dictionary"}
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertEqual(test.check_pillar("salt", dictionary="my_pillar"), ret)
        # With an ordered dict
        pillar_return = OrderedDict({"this": "dictionary"})
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertEqual(test.check_pillar("salt", dictionary="my_pillar"), ret)
        # With a string
        pillar_return = "I am a pillar."
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(
                test.check_pillar("salt", dictionary="my_pillar")["result"]
            )
        # With a list
        pillar_return = ["I am a pillar."]
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(
                test.check_pillar("salt", dictionary="my_pillar")["result"]
            )
        # With a boolean
        pillar_return = True
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(
                test.check_pillar("salt", dictionary="my_pillar")["result"]
            )
        # With an int
        pillar_return = 1
        pillar_mock = MagicMock(return_value=pillar_return)
        with patch.dict(test.__salt__, {"pillar.get": pillar_mock}):
            self.assertFalse(
                test.check_pillar("salt", dictionary="my_pillar")["result"]
            )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""
from __future__ import absolute_import, print_function, unicode_literals

import salt.config
import salt.loader
import salt.states.service as service
import salt.utils.platform
from tests.support.helpers import destructiveTest, slowTest
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


def func(name):
    """
        Mock func method
    """
    return name


class ServiceTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the service state
    """

    def setup_loader_modules(self):
        return {service: {}}

    def test_running(self):
        """
            Test to verify that the service is running
        """
        ret = [
            {"comment": "", "changes": {}, "name": "salt", "result": True},
            {
                "changes": {},
                "comment": "The service salt is already running",
                "name": "salt",
                "result": True,
            },
            {
                "changes": "saltstack",
                "comment": "The service salt is already running",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "Service salt is set to start",
                "name": "salt",
                "result": None,
            },
            {
                "changes": "saltstack",
                "comment": "Started Service salt",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "The service salt is already running",
                "name": "salt",
                "result": True,
            },
            {
                "changes": "saltstack",
                "comment": "Service salt failed to start",
                "name": "salt",
                "result": False,
            },
            {
                "changes": "saltstack",
                "comment": "Started Service salt\nService masking not available on this minion",
                "name": "salt",
                "result": True,
                "warnings": [
                    "The 'unmask' argument is not supported by this platform/action"
                ],
            },
        ]

        tmock = MagicMock(return_value=True)
        fmock = MagicMock(return_value=False)
        vmock = MagicMock(return_value="salt")
        with patch.object(service, "_enabled_used_error", vmock):
            self.assertEqual(service.running("salt", enabled=1), "salt")

        with patch.object(service, "_available", fmock):
            self.assertDictEqual(service.running("salt"), ret[0])

        with patch.object(service, "_available", tmock):
            with patch.dict(service.__opts__, {"test": False}):
                with patch.dict(
                    service.__salt__,
                    {"service.enabled": tmock, "service.status": tmock},
                ):
                    self.assertDictEqual(service.running("salt"), ret[1])

                mock = MagicMock(return_value={"changes": "saltstack"})
                with patch.dict(
                    service.__salt__,
                    {
                        "service.enabled": MagicMock(side_effect=[False, True]),
                        "service.status": tmock,
                    },
                ):
                    with patch.object(service, "_enable", mock):
                        self.assertDictEqual(service.running("salt", True), ret[2])

                with patch.dict(
                    service.__salt__,
                    {
                        "service.enabled": MagicMock(side_effect=[True, False]),
                        "service.status": tmock,
                    },
                ):
                    with patch.object(service, "_disable", mock):
                        self.assertDictEqual(service.running("salt", False), ret[2])

                with patch.dict(
                    service.__salt__,
                    {
                        "service.status": MagicMock(side_effect=[False, True]),
                        "service.enabled": MagicMock(side_effect=[False, True]),
                        "service.start": MagicMock(return_value="stack"),
                    },
                ):
                    with patch.object(
                        service,
                        "_enable",
                        MagicMock(return_value={"changes": "saltstack"}),
                    ):
                        self.assertDictEqual(service.running("salt", True), ret[4])

                with patch.dict(
                    service.__salt__,
                    {
                        "service.status": MagicMock(side_effect=[False, True]),
                        "service.enabled": MagicMock(side_effect=[False, True]),
                        "service.unmask": MagicMock(side_effect=[False, True]),
                        "service.start": MagicMock(return_value="stack"),
                    },
                ):
                    with patch.object(
                        service,
                        "_enable",
                        MagicMock(return_value={"changes": "saltstack"}),
                    ):
                        self.assertDictEqual(
                            service.running("salt", True, unmask=True), ret[7]
                        )

            with patch.dict(service.__opts__, {"test": True}):
                with patch.dict(service.__salt__, {"service.status": tmock}):
                    self.assertDictEqual(service.running("salt"), ret[5])

                with patch.dict(service.__salt__, {"service.status": fmock}):
                    self.assertDictEqual(service.running("salt"), ret[3])

            with patch.dict(service.__opts__, {"test": False}):
                with patch.dict(
                    service.__salt__,
                    {
                        "service.status": MagicMock(side_effect=[False, False]),
                        "service.enabled": MagicMock(side_effecct=[True, True]),
                        "service.start": MagicMock(return_value="stack"),
                    },
                ):
                    with patch.object(
                        service,
                        "_enable",
                        MagicMock(return_value={"changes": "saltstack"}),
                    ):
                        self.assertDictEqual(service.running("salt", True), ret[6])

    def test_dead(self):
        """
            Test to ensure that the named service is dead
        """
        ret = [
            {"changes": {}, "comment": "", "name": "salt", "result": True},
            {
                "changes": "saltstack",
                "comment": "The service salt is already dead",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "Service salt is set to be killed",
                "name": "salt",
                "result": None,
            },
            {
                "changes": "saltstack",
                "comment": "Service salt was killed",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "Service salt failed to die",
                "name": "salt",
                "result": False,
            },
            {
                "changes": "saltstack",
                "comment": "The service salt is already dead",
                "name": "salt",
                "result": True,
            },
        ]
        info_mock = MagicMock(return_value={"StartType": ""})

        mock = MagicMock(return_value="salt")
        with patch.object(service, "_enabled_used_error", mock):
            self.assertEqual(service.dead("salt", enabled=1), "salt")

        tmock = MagicMock(return_value=True)
        fmock = MagicMock(return_value=False)
        with patch.object(service, "_available", fmock):
            self.assertDictEqual(service.dead("salt"), ret[0])

        with patch.object(service, "_available", tmock):
            mock = MagicMock(return_value={"changes": "saltstack"})
            with patch.dict(service.__opts__, {"test": True}):
                with patch.dict(
                    service.__salt__,
                    {
                        "service.enabled": fmock,
                        "service.stop": tmock,
                        "service.status": fmock,
                        "service.info": info_mock,
                    },
                ):
                    with patch.object(service, "_enable", mock):
                        self.assertDictEqual(service.dead("salt", True), ret[5])

                with patch.dict(
                    service.__salt__,
                    {
                        "service.enabled": tmock,
                        "service.status": tmock,
                        "service.info": info_mock,
                    },
                ):
                    self.assertDictEqual(service.dead("salt"), ret[2])

            with patch.dict(service.__opts__, {"test": False}):
                with patch.dict(
                    service.__salt__,
                    {
                        "service.enabled": fmock,
                        "service.stop": tmock,
                        "service.status": fmock,
                        "service.info": info_mock,
                    },
                ):
                    with patch.object(service, "_enable", mock):
                        self.assertDictEqual(service.dead("salt", True), ret[1])

                with patch.dict(
                    service.__salt__,
                    {
                        "service.enabled": MagicMock(side_effect=[True, True, False]),
                        "service.status": MagicMock(side_effect=[True, False, False]),
                        "service.stop": MagicMock(return_value="stack"),
                        "service.info": info_mock,
                    },
                ):
                    with patch.object(
                        service,
                        "_enable",
                        MagicMock(return_value={"changes": "saltstack"}),
                    ):
                        self.assertDictEqual(service.dead("salt", True), ret[3])

                # test an initd which a wrong status (True even if dead)
                with patch.dict(
                    service.__salt__,
                    {
                        "service.enabled": MagicMock(side_effect=[False, False, False]),
                        "service.status": MagicMock(side_effect=[True, True, True]),
                        "service.stop": MagicMock(return_value="stack"),
                        "service.info": info_mock,
                    },
                ):
                    with patch.object(service, "_disable", MagicMock(return_value={})):
                        self.assertDictEqual(service.dead("salt", False), ret[4])

    def test_dead_with_missing_service(self):
        """
        Tests the case in which a service.dead state is executed on a state
        which does not exist.

        See https://github.com/saltstack/salt/issues/37511
        """
        name = "thisisnotarealservice"
        with patch.dict(
            service.__salt__, {"service.available": MagicMock(return_value=False)}
        ):
            ret = service.dead(name=name)
            self.assertDictEqual(
                ret,
                {
                    "changes": {},
                    "comment": "The named service {0} is not available".format(name),
                    "result": True,
                    "name": name,
                },
            )

    def test_enabled(self):
        """
            Test to verify that the service is enabled
        """
        ret = {"changes": "saltstack", "comment": "", "name": "salt", "result": True}
        mock = MagicMock(return_value={"changes": "saltstack"})
        with patch.object(service, "_enable", mock):
            self.assertDictEqual(service.enabled("salt"), ret)

    def test_disabled(self):
        """
            Test to verify that the service is disabled
        """
        ret = {"changes": "saltstack", "comment": "", "name": "salt", "result": True}
        mock = MagicMock(return_value={"changes": "saltstack"})
        with patch.object(service, "_disable", mock):
            self.assertDictEqual(service.disabled("salt"), ret)

    def test_mod_watch(self):
        """
            Test to the service watcher, called to invoke the watch command.
        """
        ret = [
            {
                "changes": {},
                "comment": "Service is already stopped",
                "name": "salt",
                "result": True,
            },
            {
                "changes": {},
                "comment": "Unable to trigger watch for service.stack",
                "name": "salt",
                "result": False,
            },
            {
                "changes": {},
                "comment": "Service is set to be started",
                "name": "salt",
                "result": None,
            },
            {
                "changes": {"salt": "salt"},
                "comment": "Service started",
                "name": "salt",
                "result": "salt",
            },
        ]

        mock = MagicMock(return_value=False)
        with patch.dict(service.__salt__, {"service.status": mock}):
            self.assertDictEqual(service.mod_watch("salt", "dead"), ret[0])

            with patch.dict(service.__salt__, {"service.start": func}):
                with patch.dict(service.__opts__, {"test": True}):
                    self.assertDictEqual(service.mod_watch("salt", "running"), ret[2])

                with patch.dict(service.__opts__, {"test": False}):
                    self.assertDictEqual(service.mod_watch("salt", "running"), ret[3])

            self.assertDictEqual(service.mod_watch("salt", "stack"), ret[1])


@destructiveTest
@skipIf(salt.utils.platform.is_darwin(), "service.running is currently failing on OSX")
class ServiceTestCaseFunctional(TestCase, LoaderModuleMockMixin):
    """
        Validate the service state
    """

    def setup_loader_modules(self):
        self.opts = salt.config.DEFAULT_MINION_OPTS.copy()
        self.opts["grains"] = salt.loader.grains(self.opts)
        self.utils = salt.loader.utils(self.opts)
        self.modules = salt.loader.minion_mods(self.opts, utils=self.utils)

        self.service_name = "cron"
        cmd_name = "crontab"
        os_family = self.opts["grains"]["os_family"]
        os_release = self.opts["grains"]["osrelease"]
        if os_family == "RedHat":
            self.service_name = "crond"
        elif os_family == "Arch":
            self.service_name = "sshd"
            cmd_name = "systemctl"
        elif os_family == "MacOS":
            self.service_name = "org.ntp.ntpd"
            if int(os_release.split(".")[1]) >= 13:
                self.service_name = "com.openssh.sshd"
        elif os_family == "Windows":
            self.service_name = "Spooler"

        if os_family != "Windows" and salt.utils.path.which(cmd_name) is None:
            self.skipTest("{0} is not installed".format(cmd_name))

        return {
            service: {
                "__grains__": self.opts["grains"],
                "__opts__": self.opts,
                "__salt__": self.modules,
                "__utils__": self.utils,
            },
        }

    def setUp(self):
        self.pre_srv_enabled = (
            True
            if self.service_name in self.modules["service.get_enabled"]()
            else False
        )
        self.post_srv_disable = False
        if not self.pre_srv_enabled:
            self.modules["service.enable"](self.service_name)
            self.post_srv_disable = True

    def tearDown(self):
        if self.post_srv_disable:
            self.modules["service.disable"](self.service_name)

    @slowTest
    def test_running_with_reload(self):
        with patch.dict(service.__opts__, {"test": False}):
            service.dead(self.service_name, enable=False)
            result = service.running(name=self.service_name, enable=True, reload=False)

        expected = {
            "changes": {self.service_name: True},
            "comment": "Service {0} has been enabled, and is "
            "running".format(self.service_name),
            "name": self.service_name,
            "result": True,
        }
        self.assertDictEqual(result, expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.serverdensity_device as serverdensity_device

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ServerdensityDeviceTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.serverdensity_device
    """

    def setup_loader_modules(self):
        return {serverdensity_device: {}}

    # 'monitored' function tests: 1

    def test_monitored(self):
        """
        Test to device is monitored with Server Density.
        """
        name = "my_special_server"

        ret = {"name": name, "changes": {}, "result": True, "comment": ""}

        mock_dict = MagicMock(return_value={"id": name})
        mock_t = MagicMock(side_effect=[True, {"agentKey": True}, [{"agentKey": True}]])
        mock_sd = MagicMock(side_effect=[["sd-agent"], [], True])
        with patch.multiple(
            serverdensity_device,
            __salt__={
                "status.all_status": mock_dict,
                "grains.items": mock_dict,
                "serverdensity_device.ls": mock_t,
                "pkg.list_pkgs": mock_sd,
                "serverdensity_device.install_agent": mock_sd,
            },
            __opts__={"test": False},
        ):
            comt = (
                "Such server name already exists in this"
                " Server Density account. And sd-agent is installed"
            )
            ret.update({"comment": comt})
            self.assertDictEqual(serverdensity_device.monitored(name), ret)

            comt = (
                "Successfully installed agent and created"
                " device in Server Density db."
            )
            ret.update(
                {
                    "comment": comt,
                    "changes": {
                        "created_device": {"agentKey": True},
                        "installed_agent": True,
                    },
                }
            )
            self.assertDictEqual(serverdensity_device.monitored(name), ret)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.nilrt_ip as nilrt_ip

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import patch
from tests.support.unit import TestCase


class NilrtIPTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.nilrt_ip module
    """

    def setup_loader_modules(self):
        return {nilrt_ip: {"__grains__": {"lsb_distrib_id": "not_nilrt"}}}

    def test_change_state_down_state(self):
        """
        Tests _change_state when not connected
        and new state is down
        """
        with patch("salt.modules.nilrt_ip._interface_to_service", return_value=True):
            with patch("salt.modules.nilrt_ip._connected", return_value=False):
                assert nilrt_ip._change_state("test_interface", "down")

    def test_change_state_up_state(self):
        """
        Tests _change_state when connected
        and new state is up
        """
        with patch("salt.modules.nilrt_ip._interface_to_service", return_value=True):
            with patch("salt.modules.nilrt_ip._connected", return_value=True):
                assert nilrt_ip._change_state("test_interface", "up")



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.states.timezone as timezone

# Import Salt Libs
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class TimezoneTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the timezone state
    """

    def setup_loader_modules(self):
        return {timezone: {}}

    def test_system(self):
        """
            Test to set the timezone for the system.
        """
        ret = {"name": "salt", "changes": {}, "result": True, "comment": ""}
        mock = MagicMock(side_effect=[CommandExecutionError, True, True, True])
        mock1 = MagicMock(side_effect=["local", "localtime", "localtime"])
        mock2 = MagicMock(return_value=False)
        with patch.dict(
            timezone.__salt__,
            {
                "timezone.zone_compare": mock,
                "timezone.get_hwclock": mock1,
                "timezone.set_hwclock": mock2,
            },
        ):
            ret.update(
                {
                    "comment": "Unable to compare desired timezone"
                    " 'salt' to system timezone: ",
                    "result": False,
                }
            )
            self.assertDictEqual(timezone.system("salt"), ret)

            ret.update(
                {
                    "comment": "Timezone salt already set," " UTC already set to salt",
                    "result": True,
                }
            )
            self.assertDictEqual(timezone.system("salt"), ret)

            with patch.dict(timezone.__opts__, {"test": True}):
                ret.update({"comment": "UTC needs to be set to True", "result": None})
                self.assertDictEqual(timezone.system("salt"), ret)

            with patch.dict(timezone.__opts__, {"test": False}):
                ret.update({"comment": "Failed to set UTC to True", "result": False})
                self.assertDictEqual(timezone.system("salt"), ret)
